import sys
import typing
import bpy


def change_effect_input(swap: typing.Union[int, str] = 'A_B'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param swap: Swap, The effect inputs to swap 
    :type swap: typing.Union[int, str]
    '''

    pass


def change_effect_type(type: typing.Union[int, str] = 'ALPHA_OVER'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: Type, Sequencer effect typeCROSS Crossfade, Crossfade effect strip type.ADD Add, Add effect strip type.SUBTRACT Subtract, Subtract effect strip type.ALPHA_OVER Alpha Over, Alpha Over effect strip type.ALPHA_UNDER Alpha Under, Alpha Under effect strip type.GAMMA_CROSS Gamma Cross, Gamma Cross effect strip type.MULTIPLY Multiply, Multiply effect strip type.OVER_DROP Alpha Over Drop, Alpha Over Drop effect strip type.WIPE Wipe, Wipe effect strip type.GLOW Glow, Glow effect strip type.TRANSFORM Transform, Transform effect strip type.COLOR Color, Color effect strip type.SPEED Speed, Color effect strip type.MULTICAM Multicam Selector.ADJUSTMENT Adjustment Layer.GAUSSIAN_BLUR Gaussian Blur.TEXT Text.COLORMIX Color Mix. 
    :type type: typing.Union[int, str]
    '''

    pass


def change_path(filepath: str = "",
                directory: str = "",
                files: typing.List['bpy.types.OperatorFileListElement'] = None,
                filter_blender: bool = False,
                filter_backup: bool = False,
                filter_image: bool = False,
                filter_movie: bool = False,
                filter_python: bool = False,
                filter_font: bool = False,
                filter_sound: bool = False,
                filter_text: bool = False,
                filter_btx: bool = False,
                filter_collada: bool = False,
                filter_alembic: bool = False,
                filter_folder: bool = True,
                filter_blenlib: bool = False,
                filemode: int = 9,
                relative_path: bool = True,
                display_type: typing.Union[int, str] = 'DEFAULT',
                sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
                use_placeholders: bool = False):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param directory: Directory, Directory of the file 
    :type directory: str
    :param files: Files 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param use_placeholders: Use Placeholders, Use placeholders for missing frames of the strip 
    :type use_placeholders: bool
    '''

    pass


def copy():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def crossfade_sounds():
    '''Do cross-fading volume animation of two selected sound strips 

    '''

    pass


def cut(frame: int = 0,
        type: typing.Union[int, str] = 'SOFT',
        side: typing.Union[int, str] = 'MOUSE'):
    '''Cut the selected strips 

    :param frame: Frame, Frame where selected strips will be cut 
    :type frame: int
    :param type: Type, The type of cut operation to perform on strips 
    :type type: typing.Union[int, str]
    :param side: Side, The side that remains selected after cutting 
    :type side: typing.Union[int, str]
    '''

    pass


def cut_multicam(camera: int = 1):
    '''Cut multi-cam strip and select camera 

    :param camera: Camera 
    :type camera: int
    '''

    pass


def deinterlace_selected_movies():
    '''Deinterlace all selected movie sources 

    '''

    pass


def delete():
    '''Erase selected strips from the sequencer 

    '''

    pass


def duplicate(mode: typing.Union[int, str] = 'TRANSLATION'):
    '''Duplicate the selected strips 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    '''

    pass


def duplicate_move(SEQUENCER_OT_duplicate=None, TRANSFORM_OT_seq_slide=None):
    '''Duplicate selected strips and move them 

    :param SEQUENCER_OT_duplicate: Duplicate Strips, Duplicate the selected strips 
    :param TRANSFORM_OT_seq_slide: Sequence Slide, Slide a sequence strip in time 
    '''

    pass


def effect_strip_add(frame_start: int = 0,
                     frame_end: int = 0,
                     channel: int = 1,
                     replace_sel: bool = True,
                     overlap: bool = False,
                     type: typing.Union[int, str] = 'ALPHA_OVER',
                     color: float = (0.0, 0.0, 0.0)):
    '''Add an effect to the sequencer, most are applied on top of existing strips 

    :param frame_start: Start Frame, Start frame of the sequence strip 
    :type frame_start: int
    :param frame_end: End Frame, End frame for the color strip 
    :type frame_end: int
    :param channel: Channel, Channel to place this strip into 
    :type channel: int
    :param replace_sel: Replace Selection, Replace the current selection 
    :type replace_sel: bool
    :param overlap: Allow Overlap, Don’t correct overlap on new sequence strips 
    :type overlap: bool
    :param type: Type, Sequencer effect typeCROSS Crossfade, Crossfade effect strip type.ADD Add, Add effect strip type.SUBTRACT Subtract, Subtract effect strip type.ALPHA_OVER Alpha Over, Alpha Over effect strip type.ALPHA_UNDER Alpha Under, Alpha Under effect strip type.GAMMA_CROSS Gamma Cross, Gamma Cross effect strip type.MULTIPLY Multiply, Multiply effect strip type.OVER_DROP Alpha Over Drop, Alpha Over Drop effect strip type.WIPE Wipe, Wipe effect strip type.GLOW Glow, Glow effect strip type.TRANSFORM Transform, Transform effect strip type.COLOR Color, Color effect strip type.SPEED Speed, Color effect strip type.MULTICAM Multicam Selector.ADJUSTMENT Adjustment Layer.GAUSSIAN_BLUR Gaussian Blur.TEXT Text.COLORMIX Color Mix. 
    :type type: typing.Union[int, str]
    :param color: Color, Initialize the strip with this color (only used when type=’COLOR’) 
    :type color: float
    '''

    pass


def enable_proxies(proxy_25: bool = False,
                   proxy_50: bool = False,
                   proxy_75: bool = False,
                   proxy_100: bool = False,
                   overwrite: bool = False):
    '''Enable selected proxies on all selected Movie, Image and Meta strips 

    :param proxy_25: 25% 
    :type proxy_25: bool
    :param proxy_50: 50% 
    :type proxy_50: bool
    :param proxy_75: 75% 
    :type proxy_75: bool
    :param proxy_100: 100% 
    :type proxy_100: bool
    :param overwrite: Overwrite 
    :type overwrite: bool
    '''

    pass


def export_subtitles(filepath: str = "",
                     check_existing: bool = True,
                     filter_blender: bool = False,
                     filter_backup: bool = False,
                     filter_image: bool = False,
                     filter_movie: bool = False,
                     filter_python: bool = False,
                     filter_font: bool = False,
                     filter_sound: bool = False,
                     filter_text: bool = False,
                     filter_btx: bool = False,
                     filter_collada: bool = False,
                     filter_alembic: bool = False,
                     filter_folder: bool = True,
                     filter_blenlib: bool = False,
                     filemode: int = 8,
                     display_type: typing.Union[int, str] = 'DEFAULT',
                     sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA'):
    '''Export .srt file containing text strips 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files 
    :type check_existing: bool
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    '''

    pass


def gap_insert(frames: int = 10):
    '''Insert gap at current frame to first strips at the right, independent of selection or locked state of strips 

    :param frames: Frames, Frames to insert after current strip 
    :type frames: int
    '''

    pass


def gap_remove(all: bool = False):
    '''Remove gap at current frame to first strip at the right, independent of selection or locked state of strips 

    :param all: All Gaps, Do all gaps to right of current frame 
    :type all: bool
    '''

    pass


def image_strip_add(
        directory: str = "",
        files: typing.List['bpy.types.OperatorFileListElement'] = None,
        filter_blender: bool = False,
        filter_backup: bool = False,
        filter_image: bool = True,
        filter_movie: bool = False,
        filter_python: bool = False,
        filter_font: bool = False,
        filter_sound: bool = False,
        filter_text: bool = False,
        filter_btx: bool = False,
        filter_collada: bool = False,
        filter_alembic: bool = False,
        filter_folder: bool = True,
        filter_blenlib: bool = False,
        filemode: int = 9,
        relative_path: bool = True,
        show_multiview: bool = False,
        use_multiview: bool = False,
        display_type: typing.Union[int, str] = 'DEFAULT',
        sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
        frame_start: int = 0,
        frame_end: int = 0,
        channel: int = 1,
        replace_sel: bool = True,
        overlap: bool = False,
        use_placeholders: bool = False):
    '''Add an image or image sequence to the sequencer 

    :param directory: Directory, Directory of the file 
    :type directory: str
    :param files: Files 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param frame_start: Start Frame, Start frame of the sequence strip 
    :type frame_start: int
    :param frame_end: End Frame, End frame for the color strip 
    :type frame_end: int
    :param channel: Channel, Channel to place this strip into 
    :type channel: int
    :param replace_sel: Replace Selection, Replace the current selection 
    :type replace_sel: bool
    :param overlap: Allow Overlap, Don’t correct overlap on new sequence strips 
    :type overlap: bool
    :param use_placeholders: Use Placeholders, Use placeholders for missing frames of the strip 
    :type use_placeholders: bool
    '''

    pass


def images_separate(length: int = 1):
    '''On image sequence strips, it returns a strip for each image 

    :param length: Length, Length of each frame 
    :type length: int
    '''

    pass


def lock():
    '''Lock strips so they can’t be transformed 

    '''

    pass


def mask_strip_add(frame_start: int = 0,
                   channel: int = 1,
                   replace_sel: bool = True,
                   overlap: bool = False,
                   mask: typing.Union[int, str] = ''):
    '''Add a mask strip to the sequencer 

    :param frame_start: Start Frame, Start frame of the sequence strip 
    :type frame_start: int
    :param channel: Channel, Channel to place this strip into 
    :type channel: int
    :param replace_sel: Replace Selection, Replace the current selection 
    :type replace_sel: bool
    :param overlap: Allow Overlap, Don’t correct overlap on new sequence strips 
    :type overlap: bool
    :param mask: Mask 
    :type mask: typing.Union[int, str]
    '''

    pass


def meta_make():
    '''Group selected strips into a metastrip 

    '''

    pass


def meta_separate():
    '''Put the contents of a metastrip back in the sequencer 

    '''

    pass


def meta_toggle():
    '''Toggle a metastrip (to edit enclosed strips) 

    '''

    pass


def movie_strip_add(
        filepath: str = "",
        files: typing.List['bpy.types.OperatorFileListElement'] = None,
        filter_blender: bool = False,
        filter_backup: bool = False,
        filter_image: bool = False,
        filter_movie: bool = True,
        filter_python: bool = False,
        filter_font: bool = False,
        filter_sound: bool = False,
        filter_text: bool = False,
        filter_btx: bool = False,
        filter_collada: bool = False,
        filter_alembic: bool = False,
        filter_folder: bool = True,
        filter_blenlib: bool = False,
        filemode: int = 9,
        relative_path: bool = True,
        show_multiview: bool = False,
        use_multiview: bool = False,
        display_type: typing.Union[int, str] = 'DEFAULT',
        sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
        frame_start: int = 0,
        channel: int = 1,
        replace_sel: bool = True,
        overlap: bool = False,
        sound: bool = True,
        use_framerate: bool = True):
    '''Add a movie strip to the sequencer 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param files: Files 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param frame_start: Start Frame, Start frame of the sequence strip 
    :type frame_start: int
    :param channel: Channel, Channel to place this strip into 
    :type channel: int
    :param replace_sel: Replace Selection, Replace the current selection 
    :type replace_sel: bool
    :param overlap: Allow Overlap, Don’t correct overlap on new sequence strips 
    :type overlap: bool
    :param sound: Sound, Load sound with the movie 
    :type sound: bool
    :param use_framerate: Use Movie Framerate, Use framerate from the movie to keep sound and video in sync 
    :type use_framerate: bool
    '''

    pass


def movieclip_strip_add(frame_start: int = 0,
                        channel: int = 1,
                        replace_sel: bool = True,
                        overlap: bool = False,
                        clip: typing.Union[int, str] = ''):
    '''Add a movieclip strip to the sequencer 

    :param frame_start: Start Frame, Start frame of the sequence strip 
    :type frame_start: int
    :param channel: Channel, Channel to place this strip into 
    :type channel: int
    :param replace_sel: Replace Selection, Replace the current selection 
    :type replace_sel: bool
    :param overlap: Allow Overlap, Don’t correct overlap on new sequence strips 
    :type overlap: bool
    :param clip: Clip 
    :type clip: typing.Union[int, str]
    '''

    pass


def mute(unselected: bool = False):
    '''Mute (un)selected strips 

    :param unselected: Unselected, Mute unselected rather than selected strips 
    :type unselected: bool
    '''

    pass


def offset_clear():
    '''Clear strip offsets from the start and end frames 

    '''

    pass


def paste():
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    '''

    pass


def reassign_inputs():
    '''Reassign the inputs for the effect strip 

    '''

    pass


def rebuild_proxy():
    '''Rebuild all selected proxies and timecode indices using the job system 

    '''

    pass


def refresh_all():
    '''Refresh the sequencer editor 

    '''

    pass


def reload(adjust_length: bool = False):
    '''Reload strips in the sequencer 

    :param adjust_length: Adjust Length, Adjust length of strips to their data length 
    :type adjust_length: bool
    '''

    pass


def rendersize():
    '''Set render size and aspect from active sequence 

    '''

    pass


def sample():
    '''Use mouse to sample color in current frame 

    '''

    pass


def scene_strip_add(frame_start: int = 0,
                    channel: int = 1,
                    replace_sel: bool = True,
                    overlap: bool = False,
                    scene: typing.Union[int, str] = ''):
    '''Add a strip to the sequencer using a blender scene as a source 

    :param frame_start: Start Frame, Start frame of the sequence strip 
    :type frame_start: int
    :param channel: Channel, Channel to place this strip into 
    :type channel: int
    :param replace_sel: Replace Selection, Replace the current selection 
    :type replace_sel: bool
    :param overlap: Allow Overlap, Don’t correct overlap on new sequence strips 
    :type overlap: bool
    :param scene: Scene 
    :type scene: typing.Union[int, str]
    '''

    pass


def select(extend: bool = False,
           deselect_all: bool = False,
           linked_handle: bool = False,
           left_right: typing.Union[int, str] = 'NONE',
           linked_time: bool = False):
    '''Select a strip (last selected becomes the “active strip”) 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    :param linked_handle: Linked Handle, Select handles next to the active strip 
    :type linked_handle: bool
    :param left_right: Left/Right, Select based on the current frame side the cursor is onNONE None, Don’t do left-right selection.MOUSE Mouse, Use mouse position for selection.LEFT Left, Select left.RIGHT Right, Select right. 
    :type left_right: typing.Union[int, str]
    :param linked_time: Linked Time, Select other strips at the same time 
    :type linked_time: bool
    '''

    pass


def select_active_side(side: typing.Union[int, str] = 'BOTH'):
    '''Select strips on the nominated side of the active strip 

    :param side: Side, The side of the handle that is selected 
    :type side: typing.Union[int, str]
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Select or deselect all strips 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET',
               tweak: bool = False):
    '''Select strips using box selection 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    :param tweak: Tweak, Operator has been activated using a tweak event 
    :type tweak: bool
    '''

    pass


def select_grouped(type: typing.Union[int, str] = 'TYPE',
                   extend: bool = False,
                   use_active_channel: bool = False):
    '''Select all strips grouped by various properties 

    :param type: TypeTYPE Type, Shared strip type.TYPE_BASIC Global Type, All strips of same basic type (Graphical or Sound).TYPE_EFFECT Effect Type, Shared strip effect type (if active strip is not an effect one, select all non-effect strips).DATA Data, Shared data (scene, image, sound, etc.).EFFECT Effect, Shared effects.EFFECT_LINK Effect/Linked, Other strips affected by the active one (sharing some time, and below or effect-assigned).OVERLAP Overlap, Overlapping time. 
    :type type: typing.Union[int, str]
    :param extend: Extend, Extend selection instead of deselecting everything first 
    :type extend: bool
    :param use_active_channel: Same Channel, Only consider strips on the same channel as the active one 
    :type use_active_channel: bool
    '''

    pass


def select_handles(side: typing.Union[int, str] = 'BOTH'):
    '''Select gizmo handles on the sides of the selected strip 

    :param side: Side, The side of the handle that is selected 
    :type side: typing.Union[int, str]
    '''

    pass


def select_less():
    '''Shrink the current selection of adjacent selected strips 

    '''

    pass


def select_linked():
    '''Select all strips adjacent to the current selection 

    '''

    pass


def select_linked_pick(extend: bool = False):
    '''Select a chain of linked strips nearest to the mouse pointer 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    '''

    pass


def select_more():
    '''Select more strips adjacent to the current selection 

    '''

    pass


def slip(offset: int = 0):
    '''Trim the contents of the active strip 

    :param offset: Offset, Offset to the data of the strip 
    :type offset: int
    '''

    pass


def snap(frame: int = 0):
    '''Frame where selected strips will be snapped 

    :param frame: Frame, Frame where selected strips will be snapped 
    :type frame: int
    '''

    pass


def sound_strip_add(
        filepath: str = "",
        files: typing.List['bpy.types.OperatorFileListElement'] = None,
        filter_blender: bool = False,
        filter_backup: bool = False,
        filter_image: bool = False,
        filter_movie: bool = False,
        filter_python: bool = False,
        filter_font: bool = False,
        filter_sound: bool = True,
        filter_text: bool = False,
        filter_btx: bool = False,
        filter_collada: bool = False,
        filter_alembic: bool = False,
        filter_folder: bool = True,
        filter_blenlib: bool = False,
        filemode: int = 9,
        relative_path: bool = True,
        display_type: typing.Union[int, str] = 'DEFAULT',
        sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
        frame_start: int = 0,
        channel: int = 1,
        replace_sel: bool = True,
        overlap: bool = False,
        cache: bool = False,
        mono: bool = False):
    '''Add a sound strip to the sequencer 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param files: Files 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param frame_start: Start Frame, Start frame of the sequence strip 
    :type frame_start: int
    :param channel: Channel, Channel to place this strip into 
    :type channel: int
    :param replace_sel: Replace Selection, Replace the current selection 
    :type replace_sel: bool
    :param overlap: Allow Overlap, Don’t correct overlap on new sequence strips 
    :type overlap: bool
    :param cache: Cache, Cache the sound in memory 
    :type cache: bool
    :param mono: Mono, Merge all the sound’s channels into one 
    :type mono: bool
    '''

    pass


def strip_jump(next: bool = True, center: bool = True):
    '''Move frame to previous edit point 

    :param next: Next Strip 
    :type next: bool
    :param center: Use strip center 
    :type center: bool
    '''

    pass


def strip_modifier_add(type: typing.Union[int, str] = 'COLOR_BALANCE'):
    '''Add a modifier to the strip 

    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def strip_modifier_copy(type: typing.Union[int, str] = 'REPLACE'):
    '''Copy modifiers of the active strip to all selected strips 

    :param type: TypeREPLACE Replace, Replace modifiers in destination.APPEND Append, Append active modifiers to selected strips. 
    :type type: typing.Union[int, str]
    '''

    pass


def strip_modifier_move(name: str = "Name",
                        direction: typing.Union[int, str] = 'UP'):
    '''Move modifier up and down in the stack 

    :param name: Name, Name of modifier to remove 
    :type name: str
    :param direction: TypeUP Up, Move modifier up in the stack.DOWN Down, Move modifier down in the stack. 
    :type direction: typing.Union[int, str]
    '''

    pass


def strip_modifier_remove(name: str = "Name"):
    '''Remove a modifier from the strip 

    :param name: Name, Name of modifier to remove 
    :type name: str
    '''

    pass


def swap(side: typing.Union[int, str] = 'RIGHT'):
    '''Swap active strip with strip to the right or left 

    :param side: Side, Side of the strip to swap 
    :type side: typing.Union[int, str]
    '''

    pass


def swap_data():
    '''Swap 2 sequencer strips 

    '''

    pass


def swap_inputs():
    '''Swap the first two inputs for the effect strip 

    '''

    pass


def unlock():
    '''Unlock strips so they can be transformed 

    '''

    pass


def unmute(unselected: bool = False):
    '''Unmute (un)selected strips 

    :param unselected: Unselected, Unmute unselected rather than selected strips 
    :type unselected: bool
    '''

    pass


def view_all():
    '''View all the strips in the sequencer 

    '''

    pass


def view_all_preview():
    '''Zoom preview to fit in the area 

    '''

    pass


def view_frame():
    '''Reset viewable area to show range around current frame 

    '''

    pass


def view_ghost_border(xmin: int = 0,
                      xmax: int = 0,
                      ymin: int = 0,
                      ymax: int = 0,
                      wait_for_input: bool = True):
    '''Set the boundaries of the border used for offset-view 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def view_selected():
    '''Zoom the sequencer on the selected strips 

    '''

    pass


def view_toggle():
    '''Toggle between sequencer views (sequence, preview, both) 

    '''

    pass


def view_zoom_ratio(ratio: float = 1.0):
    '''Change zoom ratio of sequencer preview 

    :param ratio: Ratio, Zoom ratio, 1.0 is 1:1, higher is zoomed in, lower is zoomed out 
    :type ratio: float
    '''

    pass



import sys
import typing
import mathutils


def get_model_view_matrix() -> 'mathutils.Matrix':
    '''Return a copy of the model-view matrix. 

    :return:  A 4x4 view matrix. 
    '''

    pass


def get_normal_matrix() -> 'mathutils.Matrix':
    '''Return a copy of the normal matrix. 

    :return:  A 3x3 normal matrix. 
    '''

    pass


def get_projection_matrix() -> 'mathutils.Matrix':
    '''Return a copy of the projection matrix. 

    :return:  A 4x4 projection matrix. 
    '''

    pass


def load_identity():
    '''Empty stack and set to identity. 

    '''

    pass


def load_matrix(matrix: 'mathutils.Matrix'):
    '''Load a matrix into the stack. 

    :param matrix: A 4x4 matrix. 
    :type matrix: 'mathutils.Matrix'
    '''

    pass


def load_projection_matrix(matrix: 'mathutils.Matrix'):
    '''Load a projection matrix into the stack. 

    :param matrix: A 4x4 matrix. 
    :type matrix: 'mathutils.Matrix'
    '''

    pass


def multiply_matrix(matrix: 'mathutils.Matrix'):
    '''Multiply the current stack matrix. 

    :param matrix: A 4x4 matrix. 
    :type matrix: 'mathutils.Matrix'
    '''

    pass


def pop():
    '''Remove the last model-view matrix from the stack. 

    '''

    pass


def pop_projection():
    '''Remove the last projection matrix from the stack. 

    '''

    pass


def push():
    '''Add to the model-view matrix stack. 

    '''

    pass


def push_pop():
    '''Context manager to ensure balanced push/pop calls, even in the case of an error. 

    '''

    pass


def push_pop_projection():
    '''Context manager to ensure balanced push/pop calls, even in the case of an error. 

    '''

    pass


def push_projection():
    '''Add to the projection matrix stack. 

    '''

    pass


def reset():
    '''Empty stack and set to identity. 

    '''

    pass


def scale(scale: list):
    '''Scale the current stack matrix. 

    :param scale: Scale the current stack matrix. 
    :type scale: list
    '''

    pass


def scale_uniform(scale: float):
    '''

    :param scale: Scale the current stack matrix. 
    :type scale: float
    '''

    pass


def translate(offset: list):
    '''Scale the current stack matrix. 

    :param offset: Translate the current stack matrix. 
    :type offset: list
    '''

    pass



import sys
import typing


def bake_to_keyframes(frame_start: int = 1,
                      frame_end: int = 250,
                      step: int = 1):
    '''Bake rigid body transformations of selected objects to keyframes 

    :param frame_start: Start Frame, Start frame for baking 
    :type frame_start: int
    :param frame_end: End Frame, End frame for baking 
    :type frame_end: int
    :param step: Frame Step, Frame Step 
    :type step: int
    '''

    pass


def connect(con_type: typing.Union[int, str] = 'FIXED',
            pivot_type: typing.Union[int, str] = 'CENTER',
            connection_pattern: typing.Union[int, str] = 'SELECTED_TO_ACTIVE'):
    '''Create rigid body constraints between selected rigid bodies 

    :param con_type: Type, Type of generated constraintFIXED Fixed, Glue rigid bodies together.POINT Point, Constrain rigid bodies to move around common pivot point.HINGE Hinge, Restrict rigid body rotation to one axis.SLIDER Slider, Restrict rigid body translation to one axis.PISTON Piston, Restrict rigid body translation and rotation to one axis.GENERIC Generic, Restrict translation and rotation to specified axes.GENERIC_SPRING Generic Spring, Restrict translation and rotation to specified axes with springs.MOTOR Motor, Drive rigid body around or along an axis. 
    :type con_type: typing.Union[int, str]
    :param pivot_type: Location, Constraint pivot locationCENTER Center, Pivot location is between the constrained rigid bodies.ACTIVE Active, Pivot location is at the active object position.SELECTED Selected, Pivot location is at the selected object position. 
    :type pivot_type: typing.Union[int, str]
    :param connection_pattern: Connection Pattern, Pattern used to connect objectsSELECTED_TO_ACTIVE Selected to Active, Connect selected objects to the active object.CHAIN_DISTANCE Chain by Distance, Connect objects as a chain based on distance, starting at the active object. 
    :type connection_pattern: typing.Union[int, str]
    '''

    pass


def constraint_add(type: typing.Union[int, str] = 'FIXED'):
    '''Add Rigid Body Constraint to active object 

    :param type: Rigid Body Constraint TypeFIXED Fixed, Glue rigid bodies together.POINT Point, Constrain rigid bodies to move around common pivot point.HINGE Hinge, Restrict rigid body rotation to one axis.SLIDER Slider, Restrict rigid body translation to one axis.PISTON Piston, Restrict rigid body translation and rotation to one axis.GENERIC Generic, Restrict translation and rotation to specified axes.GENERIC_SPRING Generic Spring, Restrict translation and rotation to specified axes with springs.MOTOR Motor, Drive rigid body around or along an axis. 
    :type type: typing.Union[int, str]
    '''

    pass


def constraint_remove():
    '''Remove Rigid Body Constraint from Object 

    '''

    pass


def mass_calculate(material: typing.Union[int, str] = 'DEFAULT',
                   density: float = 1.0):
    '''Automatically calculate mass values for Rigid Body Objects based on volume 

    :param material: Material Preset, Type of material that objects are made of (determines material density) 
    :type material: typing.Union[int, str]
    :param density: Density, Custom density value (kg/m^3) to use instead of material preset 
    :type density: float
    '''

    pass


def object_add(type: typing.Union[int, str] = 'ACTIVE'):
    '''Add active object as Rigid Body 

    :param type: Rigid Body TypeACTIVE Active, Object is directly controlled by simulation results.PASSIVE Passive, Object is directly controlled by animation system. 
    :type type: typing.Union[int, str]
    '''

    pass


def object_remove():
    '''Remove Rigid Body settings from Object 

    '''

    pass


def object_settings_copy():
    '''Copy Rigid Body settings from active object to selected 

    '''

    pass


def objects_add(type: typing.Union[int, str] = 'ACTIVE'):
    '''Add selected objects as Rigid Bodies 

    :param type: Rigid Body TypeACTIVE Active, Object is directly controlled by simulation results.PASSIVE Passive, Object is directly controlled by animation system. 
    :type type: typing.Union[int, str]
    '''

    pass


def objects_remove():
    '''Remove selected objects from Rigid Body simulation 

    '''

    pass


def shape_change(type: typing.Union[int, str] = 'MESH'):
    '''Change collision shapes for selected Rigid Body Objects 

    :param type: Rigid Body ShapeBOX Box, Box-like shapes (i.e. cubes), including planes (i.e. ground planes).SPHERE Sphere.CAPSULE Capsule.CYLINDER Cylinder.CONE Cone.CONVEX_HULL Convex Hull, A mesh-like surface encompassing (i.e. shrinkwrap over) all vertices (best results with fewer vertices).MESH Mesh, Mesh consisting of triangles only, allowing for more detailed interactions than convex hulls. 
    :type type: typing.Union[int, str]
    '''

    pass


def world_add():
    '''Add Rigid Body simulation world to the current scene 

    '''

    pass


def world_remove():
    '''Remove Rigid Body simulation world from the current scene 

    '''

    pass



import sys
import typing


def delete_metaelems():
    '''Delete selected metaelement(s) 

    '''

    pass


def duplicate_metaelems():
    '''Duplicate selected metaelement(s) 

    '''

    pass


def duplicate_move(MBALL_OT_duplicate_metaelems=None,
                   TRANSFORM_OT_translate=None):
    '''Make copies of the selected metaelements and move them 

    :param MBALL_OT_duplicate_metaelems: Duplicate Metaelements, Duplicate selected metaelement(s) 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def hide_metaelems(unselected: bool = False):
    '''Hide (un)selected metaelement(s) 

    :param unselected: Unselected, Hide unselected rather than selected 
    :type unselected: bool
    '''

    pass


def reveal_metaelems(select: bool = True):
    '''Reveal all hidden metaelements 

    :param select: Select 
    :type select: bool
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection of all meta elements 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_random_metaelems(percent: float = 50.0,
                            seed: int = 0,
                            action: typing.Union[int, str] = 'SELECT'):
    '''Randomly select metaelements 

    :param percent: Percent, Percentage of objects to select randomly 
    :type percent: float
    :param seed: Random Seed, Seed for the random number generator 
    :type seed: int
    :param action: Action, Selection action to executeSELECT Select, Select all elements.DESELECT Deselect, Deselect all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_similar(type: typing.Union[int, str] = 'TYPE',
                   threshold: float = 0.1):
    '''Select similar metaballs by property types 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param threshold: Threshold 
    :type threshold: float
    '''

    pass



import sys
import typing
import bpy


def ply(filepath: str = "",
        files: typing.List['bpy.types.OperatorFileListElement'] = None,
        directory: str = "",
        filter_glob: str = "*.ply"):
    '''Load a PLY geometry file 

    :param filepath: File Path, Filepath used for importing the file 
    :type filepath: str
    :param files: File Path, File path used for importing the PLY file 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param directory: directory 
    :type directory: str
    :param filter_glob: filter_glob 
    :type filter_glob: str
    '''

    pass


def stl(filepath: str = "",
        filter_glob: str = "*.stl",
        files: typing.List['bpy.types.OperatorFileListElement'] = None,
        directory: str = "",
        global_scale: float = 1.0,
        use_scene_unit: bool = False,
        use_facet_normal: bool = False,
        axis_forward: typing.Union[int, str] = 'Y',
        axis_up: typing.Union[int, str] = 'Z'):
    '''Load STL triangle mesh data 

    :param filepath: File Path, Filepath used for importing the file 
    :type filepath: str
    :param filter_glob: filter_glob 
    :type filter_glob: str
    :param files: File Path 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param directory: directory 
    :type directory: str
    :param global_scale: Scale 
    :type global_scale: float
    :param use_scene_unit: Scene Unit, Apply current scene’s unit (as defined by unit scale) to imported data 
    :type use_scene_unit: bool
    :param use_facet_normal: Facet Normals, Use (import) facet normals (note that this will still give flat shading) 
    :type use_facet_normal: bool
    :param axis_forward: Forward 
    :type axis_forward: typing.Union[int, str]
    :param axis_up: Up 
    :type axis_up: typing.Union[int, str]
    '''

    pass



import sys
import typing
import bpy


def add_render_slot():
    '''Add a new render slot 

    '''

    pass


def change_frame(frame: int = 0):
    '''Interactively change the current frame number 

    :param frame: Frame 
    :type frame: int
    '''

    pass


def clear_render_border():
    '''Clear the boundaries of the render region and disable render region 

    '''

    pass


def clear_render_slot():
    '''Clear the currently selected render slot 

    '''

    pass


def curves_point_set(point: typing.Union[int, str] = 'BLACK_POINT',
                     size: int = 1):
    '''Set black point or white point for curves 

    :param point: Point, Set black point or white point for curves 
    :type point: typing.Union[int, str]
    :param size: Sample Size 
    :type size: int
    '''

    pass


def cycle_render_slot(reverse: bool = False):
    '''Cycle through all non-void render slots 

    :param reverse: Cycle in Reverse 
    :type reverse: bool
    '''

    pass


def external_edit(filepath: str = ""):
    '''Edit image in an external application 

    :param filepath: filepath 
    :type filepath: str
    '''

    pass


def invert(invert_r: bool = False,
           invert_g: bool = False,
           invert_b: bool = False,
           invert_a: bool = False):
    '''Invert image’s channels 

    :param invert_r: Red, Invert Red Channel 
    :type invert_r: bool
    :param invert_g: Green, Invert Green Channel 
    :type invert_g: bool
    :param invert_b: Blue, Invert Blue Channel 
    :type invert_b: bool
    :param invert_a: Alpha, Invert Alpha Channel 
    :type invert_a: bool
    '''

    pass


def match_movie_length():
    '''Set image’s user’s length to the one of this video 

    '''

    pass


def new(name: str = "Untitled",
        width: int = 1024,
        height: int = 1024,
        color: float = (0.0, 0.0, 0.0, 1.0),
        alpha: bool = True,
        generated_type: typing.Union[int, str] = 'BLANK',
        float: bool = False,
        use_stereo_3d: bool = False):
    '''Create a new image 

    :param name: Name, Image data-block name 
    :type name: str
    :param width: Width, Image width 
    :type width: int
    :param height: Height, Image height 
    :type height: int
    :param color: Color, Default fill color 
    :type color: float
    :param alpha: Alpha, Create an image with an alpha channel 
    :type alpha: bool
    :param generated_type: Generated Type, Fill the image with a grid for UV map testingBLANK Blank, Generate a blank image.UV_GRID UV Grid, Generated grid to test UV mappings.COLOR_GRID Color Grid, Generated improved UV grid to test UV mappings. 
    :type generated_type: typing.Union[int, str]
    :param float: 32 bit Float, Create image with 32 bit floating point bit depth 
    :type float: bool
    :param use_stereo_3d: Stereo 3D, Create an image with left and right views 
    :type use_stereo_3d: bool
    '''

    pass


def open(filepath: str = "",
         directory: str = "",
         files: typing.List['bpy.types.OperatorFileListElement'] = None,
         filter_blender: bool = False,
         filter_backup: bool = False,
         filter_image: bool = True,
         filter_movie: bool = True,
         filter_python: bool = False,
         filter_font: bool = False,
         filter_sound: bool = False,
         filter_text: bool = False,
         filter_btx: bool = False,
         filter_collada: bool = False,
         filter_alembic: bool = False,
         filter_folder: bool = True,
         filter_blenlib: bool = False,
         filemode: int = 9,
         relative_path: bool = True,
         show_multiview: bool = False,
         use_multiview: bool = False,
         display_type: typing.Union[int, str] = 'DEFAULT',
         sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
         use_sequence_detection: bool = True):
    '''Open image 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param directory: Directory, Directory of the file 
    :type directory: str
    :param files: Files 
    :type files: typing.List['bpy.types.OperatorFileListElement']
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param use_sequence_detection: Detect Sequences, Automatically detect animated sequences in selected images (based on file names) 
    :type use_sequence_detection: bool
    '''

    pass


def pack():
    '''Pack an image as embedded data into the .blend file 

    '''

    pass


def project_apply():
    '''Project edited image back onto the object 

    '''

    pass


def project_edit():
    '''Edit a snapshot of the view-port in an external image editor 

    '''

    pass


def read_viewlayers():
    '''Read all the current scene’s view layers from cache, as needed 

    '''

    pass


def reload():
    '''Reload current image from disk 

    '''

    pass


def remove_render_slot():
    '''Remove the current render slot 

    '''

    pass


def render_border(xmin: int = 0,
                  xmax: int = 0,
                  ymin: int = 0,
                  ymax: int = 0,
                  wait_for_input: bool = True):
    '''Set the boundaries of the render region and enable render region 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def replace(filepath: str = "",
            filter_blender: bool = False,
            filter_backup: bool = False,
            filter_image: bool = True,
            filter_movie: bool = True,
            filter_python: bool = False,
            filter_font: bool = False,
            filter_sound: bool = False,
            filter_text: bool = False,
            filter_btx: bool = False,
            filter_collada: bool = False,
            filter_alembic: bool = False,
            filter_folder: bool = True,
            filter_blenlib: bool = False,
            filemode: int = 9,
            relative_path: bool = True,
            show_multiview: bool = False,
            use_multiview: bool = False,
            display_type: typing.Union[int, str] = 'DEFAULT',
            sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA'):
    '''Replace current image by another one from disk 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    '''

    pass


def sample(size: int = 1):
    '''Use mouse to sample a color in current image 

    :param size: Sample Size 
    :type size: int
    '''

    pass


def sample_line(xstart: int = 0,
                xend: int = 0,
                ystart: int = 0,
                yend: int = 0,
                cursor: int = 1002):
    '''Sample a line and show it in Scope panels 

    :param xstart: X Start 
    :type xstart: int
    :param xend: X End 
    :type xend: int
    :param ystart: Y Start 
    :type ystart: int
    :param yend: Y End 
    :type yend: int
    :param cursor: Cursor, Mouse cursor style to use during the modal operator 
    :type cursor: int
    '''

    pass


def save():
    '''Save the image with current name and settings 

    '''

    pass


def save_all_modified():
    '''Save all modified images 

    '''

    pass


def save_as(save_as_render: bool = False,
            copy: bool = False,
            filepath: str = "",
            check_existing: bool = True,
            filter_blender: bool = False,
            filter_backup: bool = False,
            filter_image: bool = True,
            filter_movie: bool = True,
            filter_python: bool = False,
            filter_font: bool = False,
            filter_sound: bool = False,
            filter_text: bool = False,
            filter_btx: bool = False,
            filter_collada: bool = False,
            filter_alembic: bool = False,
            filter_folder: bool = True,
            filter_blenlib: bool = False,
            filemode: int = 9,
            relative_path: bool = True,
            show_multiview: bool = False,
            use_multiview: bool = False,
            display_type: typing.Union[int, str] = 'DEFAULT',
            sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA'):
    '''Save the image with another name and/or settings 

    :param save_as_render: Save As Render, Apply render part of display transform when saving byte image 
    :type save_as_render: bool
    :param copy: Copy, Create a new image file without modifying the current image in blender 
    :type copy: bool
    :param filepath: File Path, Path to file 
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files 
    :type check_existing: bool
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param relative_path: Relative Path, Select the file relative to the blend file 
    :type relative_path: bool
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    '''

    pass


def save_sequence():
    '''Save a sequence of images 

    '''

    pass


def unpack(method: typing.Union[int, str] = 'USE_LOCAL', id: str = ""):
    '''Save an image packed in the .blend file to disk 

    :param method: Method, How to unpack 
    :type method: typing.Union[int, str]
    :param id: Image Name, Image data-block name to unpack 
    :type id: str
    '''

    pass


def view_all(fit_view: bool = False):
    '''View the entire image 

    :param fit_view: Fit View, Fit frame to the viewport 
    :type fit_view: bool
    '''

    pass


def view_ndof():
    '''Use a 3D mouse device to pan/zoom the view 

    '''

    pass


def view_pan(offset: float = (0.0, 0.0)):
    '''Pan the view 

    :param offset: Offset, Offset in floating point units, 1.0 is the width and height of the image 
    :type offset: float
    '''

    pass


def view_selected():
    '''View all selected UVs 

    '''

    pass


def view_zoom(factor: float = 0.0, use_cursor_init: bool = True):
    '''Zoom in/out the image 

    :param factor: Factor, Zoom factor, values higher than 1.0 zoom in, lower values zoom out 
    :type factor: float
    :param use_cursor_init: Cursor Init, Use initial cursor position 
    :type use_cursor_init: bool
    '''

    pass


def view_zoom_border(xmin: int = 0,
                     xmax: int = 0,
                     ymin: int = 0,
                     ymax: int = 0,
                     wait_for_input: bool = True,
                     zoom_out: bool = False):
    '''Zoom in the view to the nearest item contained in the border 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param zoom_out: Zoom Out 
    :type zoom_out: bool
    '''

    pass


def view_zoom_in(location: float = (0.0, 0.0)):
    '''Zoom in the image (centered around 2D cursor) 

    :param location: Location, Cursor location in screen coordinates 
    :type location: float
    '''

    pass


def view_zoom_out(location: float = (0.0, 0.0)):
    '''Zoom out the image (centered around 2D cursor) 

    :param location: Location, Cursor location in screen coordinates 
    :type location: float
    '''

    pass


def view_zoom_ratio(ratio: float = 0.0):
    '''Set zoom ratio of the view 

    :param ratio: Ratio, Zoom ratio, 1.0 is 1:1, higher is zoomed in, lower is zoomed out 
    :type ratio: float
    '''

    pass



import sys
import typing
from . import types
from . import ops
from . import app
from . import props
from . import utils
from . import path
from . import context

context: 'types.Context' = None

data: 'types.BlendData' = None
'''Access to Blenders internal data '''



import sys
import typing
import bpy


def cyclic_toggle(direction: typing.Union[int, str] = 'CYCLIC_U'):
    '''Make active spline closed/opened loop 

    :param direction: Direction, Direction to make surface cyclic in 
    :type direction: typing.Union[int, str]
    '''

    pass


def de_select_first():
    '''(De)select first of visible part of each NURBS 

    '''

    pass


def de_select_last():
    '''(De)select last of visible part of each NURBS 

    '''

    pass


def decimate(ratio: float = 1.0):
    '''Simplify selected curves 

    :param ratio: Ratio 
    :type ratio: float
    '''

    pass


def delete(type: typing.Union[int, str] = 'VERT'):
    '''Delete selected control points or segments 

    :param type: Type, Which elements to delete 
    :type type: typing.Union[int, str]
    '''

    pass


def dissolve_verts():
    '''Delete selected control points, correcting surrounding handles 

    '''

    pass


def draw(error_threshold: float = 0.0,
         fit_method: typing.Union[int, str] = 'REFIT',
         corner_angle: float = 1.22173,
         use_cyclic: bool = True,
         stroke: typing.List['bpy.types.OperatorStrokeElement'] = None,
         wait_for_input: bool = True):
    '''Draw a freehand spline 

    :param error_threshold: Error, Error distance threshold (in object units) 
    :type error_threshold: float
    :param fit_method: Fit MethodREFIT Refit, Incrementally re-fit the curve (high quality).SPLIT Split, Split the curve until the tolerance is met (fast). 
    :type fit_method: typing.Union[int, str]
    :param corner_angle: Corner Angle 
    :type corner_angle: float
    :param use_cyclic: Cyclic 
    :type use_cyclic: bool
    :param stroke: Stroke 
    :type stroke: typing.List['bpy.types.OperatorStrokeElement']
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def duplicate():
    '''Duplicate selected control points 

    '''

    pass


def duplicate_move(CURVE_OT_duplicate=None, TRANSFORM_OT_translate=None):
    '''Duplicate curve and move 

    :param CURVE_OT_duplicate: Duplicate Curve, Duplicate selected control points 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def extrude(mode: typing.Union[int, str] = 'TRANSLATION'):
    '''Extrude selected control point(s) 

    :param mode: Mode 
    :type mode: typing.Union[int, str]
    '''

    pass


def extrude_move(CURVE_OT_extrude=None, TRANSFORM_OT_translate=None):
    '''Extrude curve and move result 

    :param CURVE_OT_extrude: Extrude, Extrude selected control point(s) 
    :param TRANSFORM_OT_translate: Move, Move selected items 
    '''

    pass


def handle_type_set(type: typing.Union[int, str] = 'AUTOMATIC'):
    '''Set type of handles for selected control points 

    :param type: Type, Spline type 
    :type type: typing.Union[int, str]
    '''

    pass


def hide(unselected: bool = False):
    '''Hide (un)selected control points 

    :param unselected: Unselected, Hide unselected rather than selected 
    :type unselected: bool
    '''

    pass


def make_segment():
    '''Join two curves by their selected ends 

    '''

    pass


def match_texture_space():
    '''Match texture space to object’s bounding box 

    '''

    pass


def normals_make_consistent(calc_length: bool = False):
    '''Recalculate the direction of selected handles 

    :param calc_length: Length, Recalculate handle length 
    :type calc_length: bool
    '''

    pass


def primitive_bezier_circle_add(radius: float = 1.0,
                                enter_editmode: bool = False,
                                align: typing.Union[int, str] = 'WORLD',
                                location: float = (0.0, 0.0, 0.0),
                                rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Bezier Circle 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_bezier_curve_add(radius: float = 1.0,
                               enter_editmode: bool = False,
                               align: typing.Union[int, str] = 'WORLD',
                               location: float = (0.0, 0.0, 0.0),
                               rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Bezier Curve 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_circle_add(radius: float = 1.0,
                               enter_editmode: bool = False,
                               align: typing.Union[int, str] = 'WORLD',
                               location: float = (0.0, 0.0, 0.0),
                               rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs Circle 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_curve_add(radius: float = 1.0,
                              enter_editmode: bool = False,
                              align: typing.Union[int, str] = 'WORLD',
                              location: float = (0.0, 0.0, 0.0),
                              rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Nurbs Curve 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def primitive_nurbs_path_add(radius: float = 1.0,
                             enter_editmode: bool = False,
                             align: typing.Union[int, str] = 'WORLD',
                             location: float = (0.0, 0.0, 0.0),
                             rotation: float = (0.0, 0.0, 0.0)):
    '''Construct a Path 

    :param radius: Radius 
    :type radius: float
    :param enter_editmode: Enter Editmode, Enter editmode when adding this object 
    :type enter_editmode: bool
    :param align: Align, The alignment of the new objectWORLD World, Align the new object to the world.VIEW View, Align the new object to the view.CURSOR 3D Cursor, Use the 3D cursor orientation for the new object. 
    :type align: typing.Union[int, str]
    :param location: Location, Location for the newly added object 
    :type location: float
    :param rotation: Rotation, Rotation for the newly added object 
    :type rotation: float
    '''

    pass


def radius_set(radius: float = 1.0):
    '''Set per-point radius which is used for bevel tapering 

    :param radius: Radius 
    :type radius: float
    '''

    pass


def reveal(select: bool = True):
    '''Reveal hidden control points 

    :param select: Select 
    :type select: bool
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''(De)select all control points 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_less():
    '''Reduce current selection by deselecting boundary elements 

    '''

    pass


def select_linked():
    '''Select all control points linked to the current selection 

    '''

    pass


def select_linked_pick(deselect: bool = False):
    '''Select all control points linked to already selected ones 

    :param deselect: Deselect, Deselect linked control points rather than selecting them 
    :type deselect: bool
    '''

    pass


def select_more():
    '''Select control points directly linked to already selected ones 

    '''

    pass


def select_next():
    '''Select control points following already selected ones along the curves 

    '''

    pass


def select_nth(nth: int = 2, skip: int = 1, offset: int = 0):
    '''Deselect every other vertex 

    :param nth: Nth Element, Skip every Nth element 
    :type nth: int
    :param skip: Skip, Number of elements to skip at once 
    :type skip: int
    :param offset: Offset, Offset from the starting point 
    :type offset: int
    '''

    pass


def select_previous():
    '''Select control points preceding already selected ones along the curves 

    '''

    pass


def select_random(percent: float = 50.0,
                  seed: int = 0,
                  action: typing.Union[int, str] = 'SELECT'):
    '''Randomly select some control points 

    :param percent: Percent, Percentage of objects to select randomly 
    :type percent: float
    :param seed: Random Seed, Seed for the random number generator 
    :type seed: int
    :param action: Action, Selection action to executeSELECT Select, Select all elements.DESELECT Deselect, Deselect all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_row():
    '''Select a row of control points including active one 

    '''

    pass


def select_similar(type: typing.Union[int, str] = 'WEIGHT',
                   compare: typing.Union[int, str] = 'EQUAL',
                   threshold: float = 0.1):
    '''Select similar curve points by property type 

    :param type: Type 
    :type type: typing.Union[int, str]
    :param compare: Compare 
    :type compare: typing.Union[int, str]
    :param threshold: Threshold 
    :type threshold: float
    '''

    pass


def separate():
    '''Separate selected points from connected unselected points into a new object 

    '''

    pass


def shade_flat():
    '''Set shading to flat 

    '''

    pass


def shade_smooth():
    '''Set shading to smooth 

    '''

    pass


def shortest_path_pick():
    '''Select shortest path between two selections 

    '''

    pass


def smooth():
    '''Flatten angles of selected points 

    '''

    pass


def smooth_radius():
    '''Interpolate radii of selected points 

    '''

    pass


def smooth_tilt():
    '''Interpolate tilt of selected points 

    '''

    pass


def smooth_weight():
    '''Interpolate weight of selected points 

    '''

    pass


def spin(center: float = (0.0, 0.0, 0.0), axis: float = (0.0, 0.0, 0.0)):
    '''Extrude selected boundary row around pivot point and current view axis 

    :param center: Center, Center in global view space 
    :type center: float
    :param axis: Axis, Axis in global view space 
    :type axis: float
    '''

    pass


def spline_type_set(type: typing.Union[int, str] = 'POLY',
                    use_handles: bool = False):
    '''Set type of active spline 

    :param type: Type, Spline type 
    :type type: typing.Union[int, str]
    :param use_handles: Handles, Use handles when converting bezier curves into polygons 
    :type use_handles: bool
    '''

    pass


def spline_weight_set(weight: float = 1.0):
    '''Set softbody goal weight for selected points 

    :param weight: Weight 
    :type weight: float
    '''

    pass


def split():
    '''Split off selected points from connected unselected points 

    '''

    pass


def subdivide(number_cuts: int = 1):
    '''Subdivide selected segments 

    :param number_cuts: Number of cuts 
    :type number_cuts: int
    '''

    pass


def switch_direction():
    '''Switch direction of selected splines 

    '''

    pass


def tilt_clear():
    '''Clear the tilt of selected control points 

    '''

    pass


def vertex_add(location: float = (0.0, 0.0, 0.0)):
    '''Add a new control point (linked to only selected end-curve one, if any) 

    :param location: Location, Location to add new vertex at 
    :type location: float
    '''

    pass



import sys
import typing


def flush_edits():
    '''Flush edit data from active editing modes 

    '''

    pass


def redo():
    '''Redo previous action 

    '''

    pass


def undo():
    '''Undo previous action 

    '''

    pass


def undo_history(item: int = 0):
    '''Redo specific action in history 

    :param item: Item 
    :type item: int
    '''

    pass


def undo_push(message: str = "Addanundostep*functionmaybemoved*"):
    '''Add an undo state (internal use only) 

    :param message: Undo Message 
    :type message: str
    '''

    pass


def undo_redo():
    '''Undo and redo previous action 

    '''

    pass



import sys
import typing
import bpy


def brush_edit(stroke: typing.List['bpy.types.OperatorStrokeElement'] = None):
    '''Apply a stroke of brush to the particles 

    :param stroke: Stroke 
    :type stroke: typing.List['bpy.types.OperatorStrokeElement']
    '''

    pass


def connect_hair(all: bool = False):
    '''Connect hair to the emitter mesh 

    :param all: All hair, Connect all hair systems to the emitter mesh 
    :type all: bool
    '''

    pass


def copy_particle_systems(space: typing.Union[int, str] = 'OBJECT',
                          remove_target_particles: bool = True,
                          use_active: bool = False):
    '''Copy particle systems from the active object to selected objects 

    :param space: Space, Space transform for copying from one object to anotherOBJECT Object, Copy inside each object’s local space.WORLD World, Copy in world space. 
    :type space: typing.Union[int, str]
    :param remove_target_particles: Remove Target Particles, Remove particle systems on the target objects 
    :type remove_target_particles: bool
    :param use_active: Use Active, Use the active particle system from the context 
    :type use_active: bool
    '''

    pass


def delete(type: typing.Union[int, str] = 'PARTICLE'):
    '''Delete selected particles or keys 

    :param type: Type, Delete a full particle or only keys 
    :type type: typing.Union[int, str]
    '''

    pass


def disconnect_hair(all: bool = False):
    '''Disconnect hair from the emitter mesh 

    :param all: All hair, Disconnect all hair systems from the emitter mesh 
    :type all: bool
    '''

    pass


def duplicate_particle_system(use_duplicate_settings: bool = False):
    '''Duplicate particle system within the active object 

    :param use_duplicate_settings: Duplicate Settings, Duplicate settings as well, so the new particle system uses its own settings 
    :type use_duplicate_settings: bool
    '''

    pass


def dupliob_copy():
    '''Duplicate the current dupliobject 

    '''

    pass


def dupliob_move_down():
    '''Move dupli object down in the list 

    '''

    pass


def dupliob_move_up():
    '''Move dupli object up in the list 

    '''

    pass


def dupliob_refresh():
    '''Refresh list of dupli objects and their weights 

    '''

    pass


def dupliob_remove():
    '''Remove the selected dupliobject 

    '''

    pass


def edited_clear():
    '''Undo all edition performed on the particle system 

    '''

    pass


def hair_dynamics_preset_add(name: str = "",
                             remove_name: bool = False,
                             remove_active: bool = False):
    '''Add or remove a Hair Dynamics Preset 

    :param name: Name, Name of the preset, used to make the path name 
    :type name: str
    :param remove_name: remove_name 
    :type remove_name: bool
    :param remove_active: remove_active 
    :type remove_active: bool
    '''

    pass


def hide(unselected: bool = False):
    '''Hide selected particles 

    :param unselected: Unselected, Hide unselected rather than selected 
    :type unselected: bool
    '''

    pass


def mirror():
    '''Duplicate and mirror the selected particles along the local X axis 

    '''

    pass


def new():
    '''Add new particle settings 

    '''

    pass


def new_target():
    '''Add a new particle target 

    '''

    pass


def particle_edit_toggle():
    '''Toggle particle edit mode 

    '''

    pass


def rekey(keys_number: int = 2):
    '''Change the number of keys of selected particles (root and tip keys included) 

    :param keys_number: Number of Keys 
    :type keys_number: int
    '''

    pass


def remove_doubles(threshold: float = 0.0002):
    '''Remove selected particles close enough of others 

    :param threshold: Merge Distance, Threshold distance within which particles are removed 
    :type threshold: float
    '''

    pass


def reveal(select: bool = True):
    '''Show hidden particles 

    :param select: Select 
    :type select: bool
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''(De)select all particles’ keys 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_less():
    '''Deselect boundary selected keys of each particle 

    '''

    pass


def select_linked(deselect: bool = False, location: int = (0, 0)):
    '''Select nearest particle from mouse pointer 

    :param deselect: Deselect, Deselect linked keys rather than selecting them 
    :type deselect: bool
    :param location: Location 
    :type location: int
    '''

    pass


def select_more():
    '''Select keys linked to boundary selected keys of each particle 

    '''

    pass


def select_random(percent: float = 50.0,
                  seed: int = 0,
                  action: typing.Union[int, str] = 'SELECT',
                  type: typing.Union[int, str] = 'HAIR'):
    '''Select a randomly distributed set of hair or points 

    :param percent: Percent, Percentage of objects to select randomly 
    :type percent: float
    :param seed: Random Seed, Seed for the random number generator 
    :type seed: int
    :param action: Action, Selection action to executeSELECT Select, Select all elements.DESELECT Deselect, Deselect all elements. 
    :type action: typing.Union[int, str]
    :param type: Type, Select either hair or points 
    :type type: typing.Union[int, str]
    '''

    pass


def select_roots(action: typing.Union[int, str] = 'SELECT'):
    '''Select roots of all visible particles 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_tips(action: typing.Union[int, str] = 'SELECT'):
    '''Select tips of all visible particles 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def shape_cut():
    '''Cut hair to conform to the set shape object 

    '''

    pass


def subdivide():
    '''Subdivide selected particles segments (adds keys) 

    '''

    pass


def target_move_down():
    '''Move particle target down in the list 

    '''

    pass


def target_move_up():
    '''Move particle target up in the list 

    '''

    pass


def target_remove():
    '''Remove the selected particle target 

    '''

    pass


def unify_length():
    '''Make selected hair the same length 

    '''

    pass


def weight_set(factor: float = 1.0):
    '''Set the weight of selected keys 

    :param factor: Factor, Interpolation factor between current brush weight, and keys’ weights 
    :type factor: float
    '''

    pass



import sys
import typing


def actionzone(modifier: int = 0):
    '''Handle area action zones for mouse actions/gestures 

    :param modifier: Modifier, Modifier state 
    :type modifier: int
    '''

    pass


def animation_cancel(restore_frame: bool = True):
    '''Cancel animation, returning to the original frame 

    :param restore_frame: Restore Frame, Restore the frame when animation was initialized 
    :type restore_frame: bool
    '''

    pass


def animation_play(reverse: bool = False, sync: bool = False):
    '''Play animation 

    :param reverse: Play in Reverse, Animation is played backwards 
    :type reverse: bool
    :param sync: Sync, Drop frames to maintain framerate 
    :type sync: bool
    '''

    pass


def animation_step():
    '''Step through animation by position 

    '''

    pass


def area_dupli():
    '''Duplicate selected area into new window 

    '''

    pass


def area_join(min_x: int = -100,
              min_y: int = -100,
              max_x: int = -100,
              max_y: int = -100):
    '''Join selected areas into new window 

    :param min_x: X 1 
    :type min_x: int
    :param min_y: Y 1 
    :type min_y: int
    :param max_x: X 2 
    :type max_x: int
    :param max_y: Y 2 
    :type max_y: int
    '''

    pass


def area_move(x: int = 0, y: int = 0, delta: int = 0):
    '''Move selected area edges 

    :param x: X 
    :type x: int
    :param y: Y 
    :type y: int
    :param delta: Delta 
    :type delta: int
    '''

    pass


def area_options():
    '''Operations for splitting and merging 

    '''

    pass


def area_split(direction: typing.Union[int, str] = 'HORIZONTAL',
               factor: float = 0.5,
               cursor: int = (0, 0)):
    '''Split selected area into new windows 

    :param direction: Direction 
    :type direction: typing.Union[int, str]
    :param factor: Factor 
    :type factor: float
    :param cursor: Cursor 
    :type cursor: int
    '''

    pass


def area_swap():
    '''Swap selected areas screen positions 

    '''

    pass


def back_to_previous():
    '''Revert back to the original screen layout, before fullscreen area overlay 

    '''

    pass


def delete():
    '''Delete active screen 

    '''

    pass


def drivers_editor_show():
    '''Show drivers editor in a separate window 

    '''

    pass


def frame_jump(end: bool = False):
    '''Jump to first/last frame in frame range 

    :param end: Last Frame, Jump to the last frame of the frame range 
    :type end: bool
    '''

    pass


def frame_offset(delta: int = 0):
    '''Move current frame forward/backward by a given number 

    :param delta: Delta 
    :type delta: int
    '''

    pass


def header_toggle_menus():
    '''Expand or collapse the header pulldown menus 

    '''

    pass


def keyframe_jump(next: bool = True):
    '''Jump to previous/next keyframe 

    :param next: Next Keyframe 
    :type next: bool
    '''

    pass


def marker_jump(next: bool = True):
    '''Jump to previous/next marker 

    :param next: Next Marker 
    :type next: bool
    '''

    pass


def new():
    '''Add a new screen 

    '''

    pass


def redo_last():
    '''Display menu for last action performed 

    '''

    pass


def region_blend():
    '''Blend in and out overlapping region 

    '''

    pass


def region_context_menu():
    '''Display region context menu 

    '''

    pass


def region_flip():
    '''Toggle the region’s alignment (left/right or top/bottom) 

    '''

    pass


def region_quadview():
    '''Split selected area into camera, front, right & top views 

    '''

    pass


def region_scale():
    '''Scale selected area 

    '''

    pass


def repeat_history(index: int = 0):
    '''Display menu for previous actions performed 

    :param index: Index 
    :type index: int
    '''

    pass


def repeat_last():
    '''Repeat last action 

    '''

    pass


def screen_full_area(use_hide_panels: bool = False):
    '''Toggle display selected area as fullscreen/maximized 

    :param use_hide_panels: Hide Panels, Hide all the panels 
    :type use_hide_panels: bool
    '''

    pass


def screen_set(delta: int = 0):
    '''Cycle through available screens 

    :param delta: Delta 
    :type delta: int
    '''

    pass


def screenshot(filepath: str = "",
               check_existing: bool = True,
               filter_blender: bool = False,
               filter_backup: bool = False,
               filter_image: bool = True,
               filter_movie: bool = False,
               filter_python: bool = False,
               filter_font: bool = False,
               filter_sound: bool = False,
               filter_text: bool = False,
               filter_btx: bool = False,
               filter_collada: bool = False,
               filter_alembic: bool = False,
               filter_folder: bool = True,
               filter_blenlib: bool = False,
               filemode: int = 9,
               show_multiview: bool = False,
               use_multiview: bool = False,
               display_type: typing.Union[int, str] = 'DEFAULT',
               sort_method: typing.Union[int, str] = 'FILE_SORT_ALPHA',
               full: bool = True):
    '''Capture a picture of the active area or whole Blender window 

    :param filepath: File Path, Path to file 
    :type filepath: str
    :param check_existing: Check Existing, Check and warn on overwriting existing files 
    :type check_existing: bool
    :param filter_blender: Filter .blend files 
    :type filter_blender: bool
    :param filter_backup: Filter .blend files 
    :type filter_backup: bool
    :param filter_image: Filter image files 
    :type filter_image: bool
    :param filter_movie: Filter movie files 
    :type filter_movie: bool
    :param filter_python: Filter python files 
    :type filter_python: bool
    :param filter_font: Filter font files 
    :type filter_font: bool
    :param filter_sound: Filter sound files 
    :type filter_sound: bool
    :param filter_text: Filter text files 
    :type filter_text: bool
    :param filter_btx: Filter btx files 
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files 
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files 
    :type filter_alembic: bool
    :param filter_folder: Filter folders 
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs 
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file 
    :type filemode: int
    :param show_multiview: Enable Multi-View 
    :type show_multiview: bool
    :param use_multiview: Use Multi-View 
    :type use_multiview: bool
    :param display_type: Display TypeDEFAULT Default, Automatically determine display type for files.LIST_SHORT Short List, Display files as short list.LIST_LONG Long List, Display files as a detailed list.THUMBNAIL Thumbnails, Display files as thumbnails. 
    :type display_type: typing.Union[int, str]
    :param sort_method: File sorting modeFILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically.FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type.FILE_SORT_TIME Sort by time, Sort files by modification time.FILE_SORT_SIZE Sort by size, Sort files by size. 
    :type sort_method: typing.Union[int, str]
    :param full: Full Screen, Capture the whole window (otherwise only capture the active area) 
    :type full: bool
    '''

    pass


def space_context_cycle(direction: typing.Union[int, str] = 'NEXT'):
    '''Cycle through the editor context by activating the next/previous one 

    :param direction: Direction, Direction to cycle through 
    :type direction: typing.Union[int, str]
    '''

    pass


def space_type_set_or_cycle(space_type: typing.Union[int, str] = 'EMPTY'):
    '''Set the space type or cycle sub-type 

    :param space_type: TypeEMPTY Empty.VIEW_3D 3D Viewport, Manipulate objects in a 3D environment.IMAGE_EDITOR UV/Image Editor, View and edit images and UV Maps.NODE_EDITOR Node Editor, Editor for node-based shading and compositing tools.SEQUENCE_EDITOR Video Sequencer, Video editing tools.CLIP_EDITOR Movie Clip Editor, Motion tracking tools.DOPESHEET_EDITOR Dope Sheet, Adjust timing of keyframes.GRAPH_EDITOR Graph Editor, Edit drivers and keyframe interpolation.NLA_EDITOR Nonlinear Animation, Combine and layer Actions.TEXT_EDITOR Text Editor, Edit scripts and in-file documentation.CONSOLE Python Console, Interactive programmatic console for advanced editing and script development.INFO Info, Log of operations, warnings and error messages.TOPBAR Top Bar, Global bar at the top of the screen for global per-window settings.STATUSBAR Status Bar, Global bar at the bottom of the screen for general status information.OUTLINER Outliner, Overview of scene graph and all available data-blocks.PROPERTIES Properties, Edit properties of active object and related data-blocks.FILE_BROWSER File Browser, Browse for files and assets.PREFERENCES Preferences, Edit persistent configuration settings. 
    :type space_type: typing.Union[int, str]
    '''

    pass


def spacedata_cleanup():
    '''Remove unused settings for invisible editors 

    '''

    pass


def userpref_show():
    '''Edit user preferences and system settings 

    '''

    pass


def workspace_cycle(direction: typing.Union[int, str] = 'NEXT'):
    '''Cycle through workspaces 

    :param direction: Direction, Direction to cycle through 
    :type direction: typing.Union[int, str]
    '''

    pass



import sys
import typing


def execute_preset(filepath: str = "", menu_idname: str = ""):
    '''Execute a preset 

    :param filepath: filepath 
    :type filepath: str
    :param menu_idname: Menu ID Name, ID name of the menu this was called from 
    :type menu_idname: str
    '''

    pass


def python_file_run(filepath: str = ""):
    '''Run Python file 

    :param filepath: Path 
    :type filepath: str
    '''

    pass


def reload():
    '''Reload Scripts 

    '''

    pass



import sys
import typing


def action_set(action: typing.Union[int, str] = ''):
    '''Change the active action used 

    :param action: Action 
    :type action: typing.Union[int, str]
    '''

    pass


def animdata_operation(type: typing.Union[int, str] = 'CLEAR_ANIMDATA'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: Animation OperationCLEAR_ANIMDATA Clear Animation Data, Remove this animation data container.SET_ACT Set Action.CLEAR_ACT Unlink Action.REFRESH_DRIVERS Refresh Drivers.CLEAR_DRIVERS Clear Drivers. 
    :type type: typing.Union[int, str]
    '''

    pass


def collection_delete(hierarchy: bool = False):
    '''Delete selected collections 

    :param hierarchy: Hierarchy, Delete child objects and collections 
    :type hierarchy: bool
    '''

    pass


def collection_disable():
    '''Disable viewport drawing in the view layers 

    '''

    pass


def collection_disable_render():
    '''Do not render this collection 

    '''

    pass


def collection_drop():
    '''Drag to move to collection in Outliner 

    '''

    pass


def collection_duplicate():
    '''Recursively duplicate the collection, all its children, objects and object data 

    '''

    pass


def collection_duplicate_linked():
    '''Recursively duplicate the collection, all its children and objects, with linked object data 

    '''

    pass


def collection_enable():
    '''Enable viewport drawing in the view layers 

    '''

    pass


def collection_enable_render():
    '''Render the collection 

    '''

    pass


def collection_exclude_clear():
    '''Include collection in the active view layer 

    '''

    pass


def collection_exclude_set():
    '''Exclude collection from the active view layer 

    '''

    pass


def collection_hide():
    '''Hide the collection in this view layer 

    '''

    pass


def collection_hide_inside():
    '''Hide all the objects and collections inside the collection 

    '''

    pass


def collection_holdout_clear():
    '''Clear masking of collection in the active view layer 

    '''

    pass


def collection_holdout_set():
    '''Mask collection in the active view layer 

    '''

    pass


def collection_indirect_only_clear():
    '''Clear collection contributing only indirectly in the view layer 

    '''

    pass


def collection_indirect_only_set():
    '''Set collection to only contribute indirectly (through shadows and reflections) in the view layer 

    '''

    pass


def collection_instance():
    '''Instance selected collections to active scene 

    '''

    pass


def collection_isolate(extend: bool = False):
    '''Hide all but this collection and its parents 

    :param extend: Extend, Extend current visible collections 
    :type extend: bool
    '''

    pass


def collection_link():
    '''Link selected collections to active scene 

    '''

    pass


def collection_new(nested: bool = True):
    '''Add a new collection inside selected collection 

    :param nested: Nested, Add as child of selected collection 
    :type nested: bool
    '''

    pass


def collection_objects_deselect():
    '''Deselect objects in collection 

    '''

    pass


def collection_objects_select():
    '''Select objects in collection 

    '''

    pass


def collection_show():
    '''Show the collection in this view layer 

    '''

    pass


def collection_show_inside():
    '''Show all the objects and collections inside the collection 

    '''

    pass


def constraint_operation(type: typing.Union[int, str] = 'ENABLE'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: Constraint Operation 
    :type type: typing.Union[int, str]
    '''

    pass


def data_operation(type: typing.Union[int, str] = 'SELECT'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: Data Operation 
    :type type: typing.Union[int, str]
    '''

    pass


def drivers_add_selected():
    '''Add drivers to selected items 

    '''

    pass


def drivers_delete_selected():
    '''Delete drivers assigned to selected items 

    '''

    pass


def expanded_toggle():
    '''Expand/Collapse all items 

    '''

    pass


def hide():
    '''Hide selected objects and collections 

    '''

    pass


def highlight_update():
    '''Update the item highlight based on the current mouse position 

    '''

    pass


def id_copy():
    '''Selected data-blocks are copied to the clipboard 

    '''

    pass


def id_delete():
    '''Delete the ID under cursor 

    '''

    pass


def id_operation(type: typing.Union[int, str] = 'UNLINK'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: ID data OperationUNLINK Unlink.LOCAL Make Local.OVERRIDE_LIBRARY Add Library Override, Add a local override of this linked data-block.SINGLE Make Single User.DELETE Delete.REMAP Remap Users, Make all users of selected data-blocks to use instead current (clicked) one.COPY Copy.PASTE Paste.ADD_FAKE Add Fake User, Ensure data-block gets saved even if it isn’t in use (e.g. for motion and material libraries).CLEAR_FAKE Clear Fake User.RENAME Rename.SELECT_LINKED Select Linked. 
    :type type: typing.Union[int, str]
    '''

    pass


def id_paste():
    '''Data-blocks from the clipboard are pasted 

    '''

    pass


def id_remap(id_type: typing.Union[int, str] = 'OBJECT',
             old_id: typing.Union[int, str] = '',
             new_id: typing.Union[int, str] = ''):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param id_type: ID Type 
    :type id_type: typing.Union[int, str]
    :param old_id: Old ID, Old ID to replace 
    :type old_id: typing.Union[int, str]
    :param new_id: New ID, New ID to remap all selected IDs’ users to 
    :type new_id: typing.Union[int, str]
    '''

    pass


def item_activate(extend: bool = True,
                  recursive: bool = False,
                  deselect_all: bool = False):
    '''Handle mouse clicks to select and activate items 

    :param extend: Extend, Extend selection for activation 
    :type extend: bool
    :param recursive: Recursive, Select Objects and their children 
    :type recursive: bool
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor 
    :type deselect_all: bool
    '''

    pass


def item_drag_drop():
    '''Drag and drop element to another place 

    '''

    pass


def item_openclose(all: bool = True):
    '''Toggle whether item under cursor is enabled or closed 

    :param all: All, Close or open all items 
    :type all: bool
    '''

    pass


def item_rename():
    '''Rename item under cursor 

    '''

    pass


def keyingset_add_selected():
    '''Add selected items (blue-gray rows) to active Keying Set 

    '''

    pass


def keyingset_remove_selected():
    '''Remove selected items (blue-gray rows) from active Keying Set 

    '''

    pass


def lib_operation(type: typing.Union[int, str] = 'RENAME'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: Library OperationRENAME Rename.DELETE Delete, Delete this library and all its item from Blender - WARNING: no undo.RELOCATE Relocate, Select a new path for this library, and reload all its data.RELOAD Reload, Reload all data from this library. 
    :type type: typing.Union[int, str]
    '''

    pass


def lib_relocate():
    '''Relocate the library under cursor 

    '''

    pass


def material_drop():
    '''Drag material to object in Outliner 

    '''

    pass


def modifier_operation(type: typing.Union[int, str] = 'TOGVIS'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: Modifier Operation 
    :type type: typing.Union[int, str]
    '''

    pass


def object_operation(type: typing.Union[int, str] = 'SELECT'):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param type: Object OperationSELECT Select.DESELECT Deselect.SELECT_HIERARCHY Select Hierarchy.DELETE Delete.DELETE_HIERARCHY Delete Hierarchy.REMAP Remap Users, Make all users of selected data-blocks to use instead a new chosen one.RENAME Rename.OBJECT_MODE_ENTER Enter Mode.OBJECT_MODE_EXIT Exit Mode. 
    :type type: typing.Union[int, str]
    '''

    pass


def operation():
    '''Context menu for item operations 

    '''

    pass


def orphans_purge(num_deleted=0):
    '''Clear all orphaned data-blocks without any users from the file 

    '''

    pass


def parent_clear():
    '''Drag to clear parent in Outliner 

    '''

    pass


def parent_drop(child: str = "Object",
                parent: str = "Object",
                type: typing.Union[int, str] = 'OBJECT'):
    '''Drag to parent in Outliner 

    :param child: Child, Child Object 
    :type child: str
    :param parent: Parent, Parent Object 
    :type parent: str
    :param type: Type 
    :type type: typing.Union[int, str]
    '''

    pass


def scene_drop():
    '''Drag object to scene in Outliner 

    '''

    pass


def scene_operation(type: typing.Union[int, str] = 'DELETE'):
    '''Context menu for scene operations 

    :param type: Scene Operation 
    :type type: typing.Union[int, str]
    '''

    pass


def scroll_page(up: bool = False):
    '''Scroll page up or down 

    :param up: Up, Scroll up one page 
    :type up: bool
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Toggle the Outliner selection of items 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET'):
    '''Use box selection to select tree elements 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    '''

    pass


def show_active():
    '''Open up the tree and adjust the view so that the active Object is shown centered 

    '''

    pass


def show_hierarchy():
    '''Open all object entries and close all others 

    '''

    pass


def show_one_level(open: bool = True):
    '''Expand/collapse all entries by one level 

    :param open: Open, Expand all entries one level deep 
    :type open: bool
    '''

    pass


def unhide_all():
    '''Unhide all objects and collections 

    '''

    pass



import sys
import typing


def ndof():
    '''Use a 3D mouse device to pan/zoom the view 

    '''

    pass


def pan(deltax: int = 0, deltay: int = 0):
    '''Pan the view 

    :param deltax: Delta X 
    :type deltax: int
    :param deltay: Delta Y 
    :type deltay: int
    '''

    pass


def reset():
    '''Reset the view 

    '''

    pass


def scroll_down(deltax: int = 0, deltay: int = 0, page: bool = False):
    '''Scroll the view down 

    :param deltax: Delta X 
    :type deltax: int
    :param deltay: Delta Y 
    :type deltay: int
    :param page: Page, Scroll down one page 
    :type page: bool
    '''

    pass


def scroll_left(deltax: int = 0, deltay: int = 0):
    '''Scroll the view left 

    :param deltax: Delta X 
    :type deltax: int
    :param deltay: Delta Y 
    :type deltay: int
    '''

    pass


def scroll_right(deltax: int = 0, deltay: int = 0):
    '''Scroll the view right 

    :param deltax: Delta X 
    :type deltax: int
    :param deltay: Delta Y 
    :type deltay: int
    '''

    pass


def scroll_up(deltax: int = 0, deltay: int = 0, page: bool = False):
    '''Scroll the view up 

    :param deltax: Delta X 
    :type deltax: int
    :param deltay: Delta Y 
    :type deltay: int
    :param page: Page, Scroll up one page 
    :type page: bool
    '''

    pass


def scroller_activate():
    '''Scroll view by mouse click and drag 

    '''

    pass


def smoothview(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True):
    '''Undocumented contribute <https://developer.blender.org/T51061> 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    '''

    pass


def zoom(deltax: float = 0.0,
         deltay: float = 0.0,
         use_cursor_init: bool = True):
    '''Zoom in/out the view 

    :param deltax: Delta X 
    :type deltax: float
    :param deltay: Delta Y 
    :type deltay: float
    :param use_cursor_init: Cursor Init, Use initial cursor position 
    :type use_cursor_init: bool
    '''

    pass


def zoom_border(xmin: int = 0,
                xmax: int = 0,
                ymin: int = 0,
                ymax: int = 0,
                wait_for_input: bool = True,
                zoom_out: bool = False):
    '''Zoom in the view to the nearest item contained in the border 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param zoom_out: Zoom Out 
    :type zoom_out: bool
    '''

    pass


def zoom_in(zoomfacx: float = 0.0, zoomfacy: float = 0.0):
    '''Zoom in the view 

    :param zoomfacx: Zoom Factor X 
    :type zoomfacx: float
    :param zoomfacy: Zoom Factor Y 
    :type zoomfacy: float
    '''

    pass


def zoom_out(zoomfacx: float = 0.0, zoomfacy: float = 0.0):
    '''Zoom out the view 

    :param zoomfacx: Zoom Factor X 
    :type zoomfacx: float
    :param zoomfacy: Zoom Factor Y 
    :type zoomfacy: float
    '''

    pass



import sys
import typing


def add():
    '''Add a new time marker 

    '''

    pass


def camera_bind():
    '''Bind the selected camera to a marker on the current frame 

    '''

    pass


def delete():
    '''Delete selected time marker(s) 

    '''

    pass


def duplicate(frames: int = 0):
    '''Duplicate selected time marker(s) 

    :param frames: Frames 
    :type frames: int
    '''

    pass


def make_links_scene(scene: typing.Union[int, str] = ''):
    '''Copy selected markers to another scene 

    :param scene: Scene 
    :type scene: typing.Union[int, str]
    '''

    pass


def move(frames: int = 0, tweak: bool = False):
    '''Move selected time marker(s) 

    :param frames: Frames 
    :type frames: int
    :param tweak: Tweak, Operator has been activated using a tweak event 
    :type tweak: bool
    '''

    pass


def rename(name: str = "RenamedMarker"):
    '''Rename first selected time marker 

    :param name: Name, New name for marker 
    :type name: str
    '''

    pass


def select(extend: bool = False, camera: bool = False):
    '''Select time marker(s) 

    :param extend: Extend, Extend the selection 
    :type extend: bool
    :param camera: Camera, Select the camera 
    :type camera: bool
    '''

    pass


def select_all(action: typing.Union[int, str] = 'TOGGLE'):
    '''Change selection of all time markers 

    :param action: Action, Selection action to executeTOGGLE Toggle, Toggle selection for all elements.SELECT Select, Select all elements.DESELECT Deselect, Deselect all elements.INVERT Invert, Invert selection of all elements. 
    :type action: typing.Union[int, str]
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[int, str] = 'SET',
               tweak: bool = False):
    '''Select all time markers using box selection 

    :param xmin: X Min 
    :type xmin: int
    :param xmax: X Max 
    :type xmax: int
    :param ymin: Y Min 
    :type ymin: int
    :param ymax: Y Max 
    :type ymax: int
    :param wait_for_input: Wait for Input 
    :type wait_for_input: bool
    :param mode: ModeSET Set, Set a new selection.ADD Extend, Extend existing selection.SUB Subtract, Subtract existing selection. 
    :type mode: typing.Union[int, str]
    :param tweak: Tweak, Operator has been activated using a tweak event 
    :type tweak: bool
    '''

    pass



import sys
import typing
import bpy
import mathutils


class BMDeformVert:
    def clear(self, ):
        '''Clears all weights. 

        '''
        pass

    def get(self, key: int, default=None):
        '''Returns the deform weight matching the key or default when not found (matches pythons dictionary function of the same name). 

        :param key: The key associated with deform weight. 
        :type key: int
        :param default: Optional argument for the value to return if key is not found. 
        :type default: 
        '''
        pass

    def items(self, ) -> list:
        '''Return (group, weight) pairs for this vertex (matching pythons dict.items() functionality). 

        :rtype: list
        :return:  (key, value) pairs for each deform weight of this vertex. 
        '''
        pass

    def keys(self, ) -> list:
        '''Return the group indices used by this vertex (matching pythons dict.keys() functionality). 

        :rtype: list
        :return:  the deform group this vertex uses 
        '''
        pass

    def values(self, ) -> list:
        '''Return the weights of the deform vertex (matching pythons dict.values() functionality). 

        :rtype: list
        :return:  The weights that influence this vertex 
        '''
        pass


class BMEdge:
    '''The BMesh edge connecting 2 verts '''

    hide: bool = None
    '''Hidden state of this element. 

    :type: bool
    '''

    index: int = None
    '''Index of this element. 

    :type: int
    '''

    is_boundary: bool = None
    '''True when this edge is at the boundary of a face (read-only). 

    :type: bool
    '''

    is_contiguous: bool = None
    '''True when this edge is manifold, between two faces with the same winding (read-only). 

    :type: bool
    '''

    is_convex: bool = None
    '''True when this edge joins two convex faces, depends on a valid face normal (read-only). 

    :type: bool
    '''

    is_manifold: bool = None
    '''True when this edge is manifold (read-only). 

    :type: bool
    '''

    is_valid: bool = None
    '''True when this element is valid (hasn’t been removed). 

    :type: bool
    '''

    is_wire: bool = None
    '''True when this edge is not connected to any faces (read-only). 

    :type: bool
    '''

    link_faces: typing.List['BMFace'] = None
    '''Faces connected to this edge, (read-only). 

    :type: typing.List['BMFace']
    '''

    link_loops: typing.List['BMLoop'] = None
    '''Loops connected to this edge, (read-only). 

    :type: typing.List['BMLoop']
    '''

    seam: bool = None
    '''Seam for UV unwrapping. 

    :type: bool
    '''

    select: bool = None
    '''Selected state of this element. 

    :type: bool
    '''

    smooth: bool = None
    '''Smooth state of this element. 

    :type: bool
    '''

    tag: bool = None
    '''Generic attribute scripts can use for own logic 

    :type: bool
    '''

    verts: typing.List['BMVert'] = None
    '''Verts this edge uses (always 2), (read-only). 

    :type: typing.List['BMVert']
    '''

    def calc_face_angle(self, fallback=None) -> float:
        '''

        :param fallback: return this when the edge doesn’t have 2 faces (instead of raising a ValueError). 
        :type fallback: 
        :rtype: float
        :return:  The angle between 2 connected faces in radians. 
        '''
        pass

    def calc_face_angle_signed(self, fallback=None) -> float:
        '''

        :param fallback: return this when the edge doesn’t have 2 faces (instead of raising a ValueError). 
        :type fallback: 
        :rtype: float
        :return:  The angle between 2 connected faces in radians (negative for concave join). 
        '''
        pass

    def calc_length(self, ) -> float:
        '''

        :rtype: float
        :return:  The length between both verts. 
        '''
        pass

    def calc_tangent(self, loop: 'BMLoop') -> 'mathutils.Vector':
        '''Return the tangent at this edge relative to a face (pointing inward into the face). This uses the face normal for calculation. 

        :param loop: The loop used for tangent calculation. 
        :type loop: 'BMLoop'
        :rtype: 'mathutils.Vector'
        :return:  a normalized vector. 
        '''
        pass

    def copy_from(self, other):
        '''Copy values from another element of matching type. 

        '''
        pass

    def hide_set(self, hide: bool):
        '''Set the hide state. This is different from the hide attribute because it updates the selection and hide state of associated geometry. 

        :param hide: Hidden or visible. 
        :type hide: bool
        '''
        pass

    def normal_update(self, ):
        '''Update edges vertex normals. 

        '''
        pass

    def other_vert(self, vert: 'BMVert') -> 'BMVert':
        '''Return the other vertex on this edge or None if the vertex is not used by this edge. 

        :param vert: a vert in this edge. 
        :type vert: 'BMVert'
        :rtype: 'BMVert'
        :return:  The edges other vert. 
        '''
        pass

    def select_set(self, select: bool):
        '''Set the selection. This is different from the select attribute because it updates the selection state of associated geometry. 

        :param select: Select or de-select. 
        :type select: bool
        '''
        pass


class BMEdgeSeq:
    layers: 'BMLayerAccessEdge' = None
    '''custom-data layers (read-only). 

    :type: 'BMLayerAccessEdge'
    '''

    def ensure_lookup_table(self, ):
        '''This needs to be called again after adding/removing data in this sequence. 

        '''
        pass

    def get(self, verts: 'BMVert', fallback=None) -> 'BMEdge':
        '''Return an edge which uses the verts passed. 

        :param verts: Sequence of verts. 
        :type verts: 'BMVert'
        :param fallback: Return this value if nothing is found. 
        :type fallback: 
        :rtype: 'BMEdge'
        :return:  The edge found or None 
        '''
        pass

    def index_update(self, ):
        '''This is the equivalent of looping over all elements and assigning the index values. 

        '''
        pass

    def new(self, verts: 'BMVert', example: 'BMEdge' = None) -> 'BMEdge':
        '''Create a new edge from a given pair of verts. 

        :param verts: Vertex pair. 
        :type verts: 'BMVert'
        :param example: Existing edge to initialize settings (optional argument). 
        :type example: 'BMEdge'
        :rtype: 'BMEdge'
        :return:  The newly created edge. 
        '''
        pass

    def remove(self, edge):
        '''Remove an edge. 

        '''
        pass

    def sort(self, key=None, reverse=False):
        '''Sort the elements of this sequence, using an optional custom sort key. Indices of elements are not changed, BMElemeSeq.index_update() can be used for that. 

        :param key: The key that sets the ordering of the elements. 
        :type key: 
        :param reverse: Reverse the order of the elements 
        :type reverse: 
        '''
        pass


class BMEditSelIter:
    pass


class BMEditSelSeq:
    active: 'BMVert' = None
    '''The last selected element or None (read-only). 

    :type: 'BMVert'
    '''

    def add(self, element):
        '''Add an element to the selection history (no action taken if its already added). 

        '''
        pass

    def clear(self, ):
        '''Empties the selection history. 

        '''
        pass

    def discard(self, element):
        '''Like remove but doesn’t raise an error when the elements not in the selection list. 

        '''
        pass

    def remove(self, element):
        '''Remove an element from the selection history. 

        '''
        pass

    def validate(self, ):
        '''Ensures all elements in the selection history are selected. 

        '''
        pass


class BMElemSeq:
    '''When accessed via BMesh.verts, BMesh.edges, BMesh.faces there are also functions to create/remomove items. '''

    def index_update(self, ):
        '''This is the equivalent of looping over all elements and assigning the index values. 

        '''
        pass


class BMFace:
    '''The BMesh face with 3 or more sides '''

    edges: typing.List['BMEdge'] = None
    '''Edges of this face, (read-only). 

    :type: typing.List['BMEdge']
    '''

    hide: bool = None
    '''Hidden state of this element. 

    :type: bool
    '''

    index: int = None
    '''Index of this element. 

    :type: int
    '''

    is_valid: bool = None
    '''True when this element is valid (hasn’t been removed). 

    :type: bool
    '''

    loops: typing.List['BMLoop'] = None
    '''Loops of this face, (read-only). 

    :type: typing.List['BMLoop']
    '''

    material_index: int = None
    '''The face’s material index. 

    :type: int
    '''

    normal: 'mathutils.Vector' = None
    '''The normal for this face as a 3D, wrapped vector. 

    :type: 'mathutils.Vector'
    '''

    select: bool = None
    '''Selected state of this element. 

    :type: bool
    '''

    smooth: bool = None
    '''Smooth state of this element. 

    :type: bool
    '''

    tag: bool = None
    '''Generic attribute scripts can use for own logic 

    :type: bool
    '''

    verts: typing.List['BMVert'] = None
    '''Verts of this face, (read-only). 

    :type: typing.List['BMVert']
    '''

    def calc_area(self, ) -> float:
        '''Return the area of the face. 

        :rtype: float
        :return:  Return the area of the face. 
        '''
        pass

    def calc_center_bounds(self, ) -> 'mathutils.Vector':
        '''Return bounds center of the face. 

        :rtype: 'mathutils.Vector'
        :return:  a 3D vector. 
        '''
        pass

    def calc_center_median(self, ) -> 'mathutils.Vector':
        '''Return median center of the face. 

        :rtype: 'mathutils.Vector'
        :return:  a 3D vector. 
        '''
        pass

    def calc_center_median_weighted(self, ) -> 'mathutils.Vector':
        '''Return median center of the face weighted by edge lengths. 

        :rtype: 'mathutils.Vector'
        :return:  a 3D vector. 
        '''
        pass

    def calc_perimeter(self, ) -> float:
        '''Return the perimeter of the face. 

        :rtype: float
        :return:  Return the perimeter of the face. 
        '''
        pass

    def calc_tangent_edge(self, ) -> 'mathutils.Vector':
        '''Return face tangent based on longest edge. 

        :rtype: 'mathutils.Vector'
        :return:  a normalized vector. 
        '''
        pass

    def calc_tangent_edge_diagonal(self, ) -> 'mathutils.Vector':
        '''Return face tangent based on the edge farthest from any vertex. 

        :rtype: 'mathutils.Vector'
        :return:  a normalized vector. 
        '''
        pass

    def calc_tangent_edge_pair(self, ) -> 'mathutils.Vector':
        '''Return face tangent based on the two longest disconnected edges. 

        :rtype: 'mathutils.Vector'
        :return:  a normalized vector. 
        '''
        pass

    def calc_tangent_vert_diagonal(self, ) -> 'mathutils.Vector':
        '''Return face tangent based on the two most distent vertices. 

        :rtype: 'mathutils.Vector'
        :return:  a normalized vector. 
        '''
        pass

    def copy(self, verts: bool = True, edges: bool = True) -> 'BMFace':
        '''Make a copy of this face. 

        :param verts: When set, the faces verts will be duplicated too. 
        :type verts: bool
        :param edges: When set, the faces edges will be duplicated too. 
        :type edges: bool
        :rtype: 'BMFace'
        :return:  The newly created face. 
        '''
        pass

    def copy_from(self, other):
        '''Copy values from another element of matching type. 

        '''
        pass

    def copy_from_face_interp(self, face: 'BMFace', vert: bool = True):
        '''Interpolate the customdata from another face onto this one (faces should overlap). 

        :param face: The face to interpolate data from. 
        :type face: 'BMFace'
        :param vert: When True, also copy vertex data. 
        :type vert: bool
        '''
        pass

    def hide_set(self, hide: bool):
        '''Set the hide state. This is different from the hide attribute because it updates the selection and hide state of associated geometry. 

        :param hide: Hidden or visible. 
        :type hide: bool
        '''
        pass

    def normal_flip(self, ):
        '''Reverses winding of a face, which flips its normal. 

        '''
        pass

    def normal_update(self, ):
        '''Update face’s normal. 

        '''
        pass

    def select_set(self, select: bool):
        '''Set the selection. This is different from the select attribute because it updates the selection state of associated geometry. 

        :param select: Select or de-select. 
        :type select: bool
        '''
        pass


class BMFaceSeq:
    active: 'BMFace' = None
    '''active face. 

    :type: 'BMFace'
    '''

    layers: 'BMLayerAccessFace' = None
    '''custom-data layers (read-only). 

    :type: 'BMLayerAccessFace'
    '''

    def ensure_lookup_table(self, ):
        '''This needs to be called again after adding/removing data in this sequence. 

        '''
        pass

    def get(self, verts: 'BMVert', fallback=None) -> 'BMFace':
        '''Return a face which uses the verts passed. 

        :param verts: Sequence of verts. 
        :type verts: 'BMVert'
        :param fallback: Return this value if nothing is found. 
        :type fallback: 
        :rtype: 'BMFace'
        :return:  The face found or None 
        '''
        pass

    def index_update(self, ):
        '''This is the equivalent of looping over all elements and assigning the index values. 

        '''
        pass

    def new(self, verts: 'BMVert', example: 'BMFace' = None) -> 'BMFace':
        '''Create a new face from a given set of verts. 

        :param verts: Sequence of 3 or more verts. 
        :type verts: 'BMVert'
        :param example: Existing face to initialize settings (optional argument). 
        :type example: 'BMFace'
        :rtype: 'BMFace'
        :return:  The newly created face. 
        '''
        pass

    def remove(self, face):
        '''Remove a face. 

        '''
        pass

    def sort(self, key=None, reverse=False):
        '''Sort the elements of this sequence, using an optional custom sort key. Indices of elements are not changed, BMElemeSeq.index_update() can be used for that. 

        :param key: The key that sets the ordering of the elements. 
        :type key: 
        :param reverse: Reverse the order of the elements 
        :type reverse: 
        '''
        pass


class BMIter:
    '''Internal BMesh type for looping over verts/faces/edges, used for iterating over BMElemSeq types. '''

    pass


class BMLayerAccessEdge:
    '''Exposes custom-data layer attributes. '''

    bevel_weight: 'BMLayerCollection' = None
    '''Bevel weight float in [0 - 1]. 

    :type: 'BMLayerCollection'
    '''

    crease: 'BMLayerCollection' = None
    '''Edge crease for subsurf - float in [0 - 1]. 

    :type: 'BMLayerCollection'
    '''

    float = None
    '''type: BMLayerCollection '''

    freestyle = None
    '''type: BMLayerCollection '''

    int = None
    '''type: BMLayerCollection '''

    string = None
    '''type: BMLayerCollection '''


class BMLayerAccessFace:
    '''Exposes custom-data layer attributes. '''

    face_map = None
    '''type: BMLayerCollection '''

    float = None
    '''type: BMLayerCollection '''

    freestyle = None
    '''type: BMLayerCollection '''

    int = None
    '''type: BMLayerCollection '''

    string = None
    '''type: BMLayerCollection '''


class BMLayerAccessLoop:
    '''Exposes custom-data layer attributes. '''

    color = None
    '''type: BMLayerCollection '''

    float = None
    '''type: BMLayerCollection '''

    int = None
    '''type: BMLayerCollection '''

    string = None
    '''type: BMLayerCollection '''

    uv = None
    '''type: BMLayerCollection '''


class BMLayerAccessVert:
    '''Exposes custom-data layer attributes. '''

    bevel_weight: 'BMLayerCollection' = None
    '''Bevel weight float in [0 - 1]. 

    :type: 'BMLayerCollection'
    '''

    deform = None
    '''type: BMLayerCollection '''

    float = None
    '''type: BMLayerCollection '''

    int = None
    '''type: BMLayerCollection '''

    paint_mask = None
    '''type: BMLayerCollection '''

    shape: 'BMLayerCollection' = None
    '''Vertex shapekey absolute location (as a 3D Vector). 

    :type: 'BMLayerCollection'
    '''

    skin = None
    '''type: BMLayerCollection '''

    string = None
    '''type: BMLayerCollection '''


class BMLayerCollection:
    '''Gives access to a collection of custom-data layers of the same type and behaves like python dictionaries, except for the ability to do list like index access. '''

    active: 'BMLayerItem' = None
    '''The active layer of this type (read-only). 

    :type: 'BMLayerItem'
    '''

    is_singleton: bool = None
    '''True if there can exists only one layer of this type (read-only). 

    :type: bool
    '''

    def get(self, key: str, default=None):
        '''Returns the value of the layer matching the key or default when not found (matches pythons dictionary function of the same name). 

        :param key: The key associated with the layer. 
        :type key: str
        :param default: Optional argument for the value to return if key is not found. 
        :type default: 
        '''
        pass

    def items(self, ) -> list:
        '''Return the identifiers of collection members (matching pythons dict.items() functionality). 

        :rtype: list
        :return:  (key, value) pairs for each member of this collection. 
        '''
        pass

    def keys(self, ) -> list:
        '''Return the identifiers of collection members (matching pythons dict.keys() functionality). 

        :rtype: list
        :return:  the identifiers for each member of this collection. 
        '''
        pass

    def new(self, name: str) -> 'BMLayerItem':
        '''Create a new layer 

        :param name: Optional name argument (will be made unique). 
        :type name: str
        :rtype: 'BMLayerItem'
        :return:  The newly created layer. 
        '''
        pass

    def remove(self, layer: 'BMLayerItem'):
        '''Remove a layer 

        :param layer: The layer to remove. 
        :type layer: 'BMLayerItem'
        '''
        pass

    def values(self, ) -> list:
        '''Return the values of collection (matching pythons dict.values() functionality). 

        :rtype: list
        :return:  the members of this collection. 
        '''
        pass

    def verify(self, ) -> 'BMLayerItem':
        '''Create a new layer or return an existing active layer 

        :rtype: 'BMLayerItem'
        :return:  The newly verified layer. 
        '''
        pass


class BMLayerItem:
    '''Exposes a single custom data layer, their main purpose is for use as item accessors to custom-data when used with vert/edge/face/loop data. '''

    name: str = None
    '''The layers unique name (read-only). 

    :type: str
    '''

    def copy_from(self, other):
        '''Return a copy of the layer 

        :param other: Another layer to copy from. 
        :type other: 
        :param other: BMLayerItem 
        :type other: 
        '''
        pass


class BMLoop:
    '''This is normally accessed from BMFace.loops where each face loop represents a corner of the face. '''

    edge: 'BMEdge' = None
    '''The loop’s edge (between this loop and the next), (read-only). 

    :type: 'BMEdge'
    '''

    face: 'BMFace' = None
    '''The face this loop makes (read-only). 

    :type: 'BMFace'
    '''

    index: int = None
    '''Index of this element. 

    :type: int
    '''

    is_convex: bool = None
    '''True when this loop is at the convex corner of a face, depends on a valid face normal (read-only). 

    :type: bool
    '''

    is_valid: bool = None
    '''True when this element is valid (hasn’t been removed). 

    :type: bool
    '''

    link_loop_next: 'BMLoop' = None
    '''The next face corner (read-only). 

    :type: 'BMLoop'
    '''

    link_loop_prev: 'BMLoop' = None
    '''The previous face corner (read-only). 

    :type: 'BMLoop'
    '''

    link_loop_radial_next: 'BMLoop' = None
    '''The next loop around the edge (read-only). 

    :type: 'BMLoop'
    '''

    link_loop_radial_prev: 'BMLoop' = None
    '''The previous loop around the edge (read-only). 

    :type: 'BMLoop'
    '''

    link_loops: typing.List['BMLoop'] = None
    '''Loops connected to this loop, (read-only). 

    :type: typing.List['BMLoop']
    '''

    tag: bool = None
    '''Generic attribute scripts can use for own logic 

    :type: bool
    '''

    vert: 'BMVert' = None
    '''The loop’s vertex (read-only). 

    :type: 'BMVert'
    '''

    def calc_angle(self, ) -> float:
        '''Return the angle at this loops corner of the face. This is calculated so sharper corners give lower angles. 

        :rtype: float
        :return:  The angle in radians. 
        '''
        pass

    def calc_normal(self, ) -> 'mathutils.Vector':
        '''Return normal at this loops corner of the face. Falls back to the face normal for straight lines. 

        :rtype: 'mathutils.Vector'
        :return:  a normalized vector. 
        '''
        pass

    def calc_tangent(self, ) -> 'mathutils.Vector':
        '''Return the tangent at this loops corner of the face (pointing inward into the face). Falls back to the face normal for straight lines. 

        :rtype: 'mathutils.Vector'
        :return:  a normalized vector. 
        '''
        pass

    def copy_from(self, other):
        '''Copy values from another element of matching type. 

        '''
        pass

    def copy_from_face_interp(self,
                              face: 'BMFace',
                              vert: bool = True,
                              multires: bool = True):
        '''Interpolate the customdata from a face onto this loop (the loops vert should overlap the face). 

        :param face: The face to interpolate data from. 
        :type face: 'BMFace'
        :param vert: When enabled, interpolate the loops vertex data (optional). 
        :type vert: bool
        :param multires: When enabled, interpolate the loops multires data (optional). 
        :type multires: bool
        '''
        pass


class BMLoopSeq:
    layers: 'BMLayerAccessLoop' = None
    '''custom-data layers (read-only). 

    :type: 'BMLayerAccessLoop'
    '''


class BMLoopUV:
    pin_uv: bool = None
    '''UV pin state. 

    :type: bool
    '''

    select: bool = None
    '''UV select state. 

    :type: bool
    '''

    select_edge: bool = None
    '''UV edge select state. 

    :type: bool
    '''

    uv: 'mathutils.Vector' = None
    '''Loops UV (as a 2D Vector). 

    :type: 'mathutils.Vector'
    '''


class BMVert:
    '''The BMesh vertex type '''

    co: 'mathutils.Vector' = None
    '''The coordinates for this vertex as a 3D, wrapped vector. 

    :type: 'mathutils.Vector'
    '''

    hide: bool = None
    '''Hidden state of this element. 

    :type: bool
    '''

    index: int = None
    '''Index of this element. 

    :type: int
    '''

    is_boundary: bool = None
    '''True when this vertex is connected to boundary edges (read-only). 

    :type: bool
    '''

    is_manifold: bool = None
    '''True when this vertex is manifold (read-only). 

    :type: bool
    '''

    is_valid: bool = None
    '''True when this element is valid (hasn’t been removed). 

    :type: bool
    '''

    is_wire: bool = None
    '''True when this vertex is not connected to any faces (read-only). 

    :type: bool
    '''

    link_edges: typing.List['BMEdge'] = None
    '''Edges connected to this vertex (read-only). 

    :type: typing.List['BMEdge']
    '''

    link_faces: typing.List['BMFace'] = None
    '''Faces connected to this vertex (read-only). 

    :type: typing.List['BMFace']
    '''

    link_loops: typing.List['BMLoop'] = None
    '''Loops that use this vertex (read-only). 

    :type: typing.List['BMLoop']
    '''

    normal: 'mathutils.Vector' = None
    '''The normal for this vertex as a 3D, wrapped vector. 

    :type: 'mathutils.Vector'
    '''

    select: bool = None
    '''Selected state of this element. 

    :type: bool
    '''

    tag: bool = None
    '''Generic attribute scripts can use for own logic 

    :type: bool
    '''

    def calc_edge_angle(self, fallback=None) -> float:
        '''Return the angle between this vert’s two connected edges. 

        :param fallback: return this when the vert doesn’t have 2 edges (instead of raising a ValueError). 
        :type fallback: 
        :rtype: float
        :return:  Angle between edges in radians. 
        '''
        pass

    def calc_shell_factor(self, ) -> float:
        '''Return a multiplier calculated based on the sharpness of the vertex. Where a flat surface gives 1.0, and higher values sharper edges. This is used to maintain shell thickness when offsetting verts along their normals. 

        :rtype: float
        :return:  offset multiplier 
        '''
        pass

    def copy_from(self, other):
        '''Copy values from another element of matching type. 

        '''
        pass

    def copy_from_face_interp(self, face: 'BMFace'):
        '''Interpolate the customdata from a face onto this loop (the loops vert should overlap the face). 

        :param face: The face to interpolate data from. 
        :type face: 'BMFace'
        '''
        pass

    def copy_from_vert_interp(self, vert_pair: 'BMVert', fac):
        '''Interpolate the customdata from a vert between 2 other verts. 

        :param vert_pair: The vert to interpolate data from. 
        :type vert_pair: 'BMVert'
        '''
        pass

    def hide_set(self, hide: bool):
        '''Set the hide state. This is different from the hide attribute because it updates the selection and hide state of associated geometry. 

        :param hide: Hidden or visible. 
        :type hide: bool
        '''
        pass

    def normal_update(self, ):
        '''Update vertex normal. 

        '''
        pass

    def select_set(self, select: bool):
        '''Set the selection. This is different from the select attribute because it updates the selection state of associated geometry. 

        :param select: Select or de-select. 
        :type select: bool
        '''
        pass


class BMVertSeq:
    layers: 'BMLayerAccessVert' = None
    '''custom-data layers (read-only). 

    :type: 'BMLayerAccessVert'
    '''

    def ensure_lookup_table(self, ):
        '''This needs to be called again after adding/removing data in this sequence. 

        '''
        pass

    def index_update(self, ):
        '''This is the equivalent of looping over all elements and assigning the index values. 

        '''
        pass

    def new(self, co: float = (0.0, 0.0, 0.0),
            example: 'BMVert' = None) -> 'BMVert':
        '''Create a new vertex. 

        :param co: The initial location of the vertex (optional argument). 
        :type co: float
        :param example: Existing vert to initialize settings. 
        :type example: 'BMVert'
        :rtype: 'BMVert'
        :return:  The newly created edge. 
        '''
        pass

    def remove(self, vert):
        '''Remove a vert. 

        '''
        pass

    def sort(self, key=None, reverse=False):
        '''Sort the elements of this sequence, using an optional custom sort key. Indices of elements are not changed, BMElemeSeq.index_update() can be used for that. 

        :param key: The key that sets the ordering of the elements. 
        :type key: 
        :param reverse: Reverse the order of the elements 
        :type reverse: 
        '''
        pass


class BMesh:
    '''The BMesh data structure '''

    edges: typing.List['BMEdge'] = None
    '''This meshes edge sequence (read-only). 

    :type: typing.List['BMEdge']
    '''

    faces: typing.List['BMFace'] = None
    '''This meshes face sequence (read-only). 

    :type: typing.List['BMFace']
    '''

    is_valid: bool = None
    '''True when this element is valid (hasn’t been removed). 

    :type: bool
    '''

    is_wrapped: bool = None
    '''True when this mesh is owned by blender (typically the editmode BMesh). 

    :type: bool
    '''

    loops: typing.List['BMLoop'] = None
    '''This meshes loops (read-only). 

    :type: typing.List['BMLoop']
    '''

    select_history: list = None
    '''Sequence of selected items (the last is displayed as active). 

    :type: list
    '''

    select_mode: set = None
    '''The selection mode, values can be {‘VERT’, ‘EDGE’, ‘FACE’}, can’t be assigned an empty set. 

    :type: set
    '''

    verts: typing.List['BMVert'] = None
    '''This meshes vert sequence (read-only). 

    :type: typing.List['BMVert']
    '''

    def calc_loop_triangles(self, ) -> typing.List['BMLoop']:
        '''Calculate triangle tessellation from quads/ngons. 

        :rtype: typing.List['BMLoop']
        :return:  The triangulated faces. 
        '''
        pass

    def calc_volume(self, signed: bool = False) -> float:
        '''Calculate mesh volume based on face normals. 

        :param signed: when signed is true, negative values may be returned. 
        :type signed: bool
        :rtype: float
        :return:  The volume of the mesh. 
        '''
        pass

    def clear(self, ):
        '''Clear all mesh data. 

        '''
        pass

    def copy(self, ) -> 'BMesh':
        '''

        :rtype: 'BMesh'
        :return:  A copy of this BMesh. 
        '''
        pass

    def free(self, ):
        '''Explicitly free the BMesh data from memory, causing exceptions on further access. 

        '''
        pass

    def from_mesh(self,
                  mesh: 'bpy.types.Mesh',
                  face_normals=True,
                  use_shape_key: bool = False,
                  shape_key_index: int = 0):
        '''Initialize this bmesh from existing mesh datablock. 

        :param mesh: The mesh data to load. 
        :type mesh: 'bpy.types.Mesh'
        :param use_shape_key: Use the locations from a shape key. 
        :type use_shape_key: bool
        :param shape_key_index: The shape key index to use. 
        :type shape_key_index: int
        '''
        pass

    def from_object(self,
                    object: 'bpy.types.Object',
                    depsgraph,
                    deform: bool = True,
                    cage: bool = False,
                    face_normals: bool = True):
        '''Initialize this bmesh from existing object datablock (currently only meshes are supported). 

        :param object: The object data to load. 
        :type object: 'bpy.types.Object'
        :param deform: Apply deformation modifiers. 
        :type deform: bool
        :param cage: Get the mesh as a deformed cage. 
        :type cage: bool
        :param face_normals: Calculate face normals. 
        :type face_normals: bool
        '''
        pass

    def normal_update(self, ):
        '''Update mesh normals. 

        '''
        pass

    def select_flush(self, select: bool):
        '''Flush selection, independent of the current selection mode. 

        :param select: flush selection or de-selected elements. 
        :type select: bool
        '''
        pass

    def select_flush_mode(self, ):
        '''flush selection based on the current mode current BMesh.select_mode. 

        '''
        pass

    def to_mesh(self, mesh: 'bpy.types.Mesh'):
        '''Writes this BMesh data into an existing Mesh datablock. 

        :param mesh: The mesh data to write into. 
        :type mesh: 'bpy.types.Mesh'
        '''
        pass

    def transform(self, matrix: 'mathutils.Matrix', filter: set = None):
        '''Transform the mesh (optionally filtering flagged data only). 

        :param matrix: transform matrix. 
        :type matrix: 'mathutils.Matrix'
        :param filter: set of values in (‘SELECT’, ‘HIDE’, ‘SEAM’, ‘SMOOTH’, ‘TAG’). 
        :type filter: set
        '''
        pass



import sys
import typing


def add():
    '''Add brush by mode type 

    '''

    pass


def add_gpencil():
    '''Add brush for Grease Pencil 

    '''

    pass


def curve_preset(shape: typing.Union[int, str] = 'SMOOTH'):
    '''Set brush shape 

    :param shape: Mode 
    :type shape: typing.Union[int, str]
    '''

    pass


def reset():
    '''Return brush to defaults based on current tool 

    '''

    pass


def scale_size(scalar: float = 1.0):
    '''Change brush size by a scalar 

    :param scalar: Scalar, Factor to scale brush size by 
    :type scalar: float
    '''

    pass


def stencil_control(mode: typing.Union[int, str] = 'TRANSLATION',
                    texmode: typing.Union[int, str] = 'PRIMARY'):
    '''Control the stencil brush 

    :param mode: Tool 
    :type mode: typing.Union[int, str]
    :param texmode: Tool 
    :type texmode: typing.Union[int, str]
    '''

    pass


def stencil_fit_image_aspect(use_repeat: bool = True,
                             use_scale: bool = True,
                             mask: bool = False):
    '''When using an image texture, adjust the stencil size to fit the image aspect ratio 

    :param use_repeat: Use Repeat, Use repeat mapping values 
    :type use_repeat: bool
    :param use_scale: Use Scale, Use texture scale values 
    :type use_scale: bool
    :param mask: Modify Mask Stencil, Modify either the primary or mask stencil 
    :type mask: bool
    '''

    pass


def stencil_reset_transform(mask: bool = False):
    '''Reset the stencil transformation to the default 

    :param mask: Modify Mask Stencil, Modify either the primary or mask stencil 
    :type mask: bool
    '''

    pass



import sys
import typing


def poly_3d_calc(veclist, pt):
    '''Calculate barycentric weights for a point on a polygon. 

    :param veclist: list of vectors 
    :param pt: point :rtype: list of per-vector weights 
    '''

    pass



import sys
import typing


def color_add():
    '''Add new color to active palette 

    '''

    pass


def color_delete():
    '''Remove active color from palette 

    '''

    pass


def new():
    '''Add new palette 

    '''

    pass
